#!/usr/bin/perl
#caractère UTF8

# Modules Perl nécessaire
use FindBin::libs;
use strict;
use DBI;
use Term::ProgressBar;
use Unicode::String qw(latin1 latin1);
use Lingua::FR::Phonemise ;
use File::Copy;
use Benchmark ;
use Class::Date qw (now) ;
use XML::Simple;
use List::MoreUtils qw(uniq);
# Ce module est créé par getfile.pl qui charge les fichiers
use config ('$Wikiquotefile','$SourceDir','$FortuneURL','$LexiqueFile','$PrenomsFile','$WiktionaryFile','$FileVilleFrance','$OpenOfficeDictFile');

# quelques informations 
my $datedebut=now ;
my $VERSION="3.0";

# fichiers Datas non volatiles c'est à dire fabriqué ou modifié par mes soins
my $FileVilleBelgique="villesbelgique.txt";
my $FilePays="Pays.txt";
my $DicMotsRare="meral.xml";
my $DictSyn="thes_fr.dat";


# test à 0 écrase la base et la regénère
# test à 1 permet de modifier le code un peu après pour n'éxécuter que quelques commandes 

my $Test=0	; # permet de reprendre en cours en bougeant quelques lignes car le procédé est tellement long ...


# variables globales
my @marqueurtab;
my %marqueur ;
my $quoteouvrante=1;
my $db ;

my $DestinationBase;
my $NBLexique;
my $NBWiktionary;
my $NBCitation;
my $NBPrenom;
my $NBPays;
my $NBVilleFrance;
my $NBVilleBelge;
my $NBVERBES;
my $NBSynonymes;
my $NBMotsRares;

CheckFiles();
CheckPrograms();
$DestinationBase="dicorime.db";


print "Comptage du nombre de données\n";

$NBLexique=NombreLigne("$SourceDir/$LexiqueFile");
$NBWiktionary=NombreLigne("$SourceDir/$WiktionaryFile");
$NBCitation=52806;
$NBPrenom=NombreLigne("$SourceDir/$PrenomsFile");
$NBPays=236;
$NBVilleFrance=42000;
$NBVilleBelge=5478;
$NBVERBES=7006;
$NBMotsRares=4827;
$NBSynonymes=NombreLigne("$SourceDir/$DictSyn");

print "Mots = $NBLexique\n";
print "Définition = $NBWiktionary\n";
print "Citations = $NBCitation\n";
if  ( ! $Test ) {
unlink "$DestinationBase";
}

# Ouverture de la base 
$db = DBI->connect("dbi:SQLite:dbname=$DestinationBase","","",);
#$db->{sqlite_unicode}=1;
$db->{AutoCommit} = 0;
$db->do("PRAGMA synchronous = OFF;");
$db->do("PRAGMA cache_size = 800000 ; BEGIN TRANSACTION ");
if ( ! $Test )
{
$db->do("CREATE TABLE dico ( ID integer PRIMARY KEY, ortho TEXT, lemme TEXT, cgram TEXT, genre TEXT,nombre TEXT, nbsyll TEXT, freq INTEGER,PH TEXT, PH1 TEXT, PH2 TEXT, PH3 TEXT, PH4 TEXT, PH5 TEXT, PH6 TEXT, PH7 TEXT, PH8 TEXT)");
$db->do("CREATE TABLE citations (ID integer PRIMARY KEY,genre TEXT,rubrique TEXT,citation TEXT,auteur TEXT)");
$db->do("CREATE TABLE definitions (ID integer PRIMARY KEY,ortho TEXT,definition TEXT,origin TEXT)");
$db->do("CREATE TABLE synonymes (ID integer PRIMARY KEY,ortho TEXT,cgram TEXT,liste TEXT)");
}

# Le mode test permet de recopier quelques lignes pour n'exécuter que quelques modules partiellement 
if ( ! $Test ) {
   CreationCitations(); $db->commit; controlsizetable();
   CreationSynonymes(); $db->commit;controlsizetable();
   CreationRimes(); $db->commit;controlsizetable();
   CreationPrenoms(); $db->commit;controlsizetable();
   CreationPays(); $db->commit;controlsizetable();
   CreationVilleFrance(); $db->commit;controlsizetable();
   CreationVilleBelgique(); $db->commit;controlsizetable();
   CreationVerbiste(); $db->commit;controlsizetable();
   FormateDefinitions(0); $db->commit;controlsizetable();
   CreationMotsRares(); 
   ArrangeBaseSQL();
   CreateIndexes();
   print "commit final\n";
   $db->commit;
   my $datefin = now;
   my $temps=$datefin-$datedebut;
   print "* * * * * * * * * * \n";
   print "Temps d'exécutions\n";
   print $temps->day," jours\n",$temps->hour," heures\n", $temps->min," min\n",$temps->sec," sec\n",
}
else
{
   CreationVilleFrance();
   $db->commit;
   ArrangeBaseSQL();
   CreateIndexes();
   
}


my $compteurdefinition=0;


sub CreationPays {
	print ("Inclusion des Pays\n");
	my  $td0 = new Benchmark;
	my $compt=0;
	my $progress = Term::ProgressBar->new($NBPays);
	open ( FIC, "<$SourceDir/$FilePays");
	while ( my  $ligne=<FIC> ) 
	{
		chomp ($ligne);
		my ($capitale,$pays,$nombre,$genre)=split(/\|/,$ligne);
		InsereDico($pays,lc($pays),Text2Phoneme(lc($pays)),Cgram("PAY"),$nombre,$genre,2);
		InsereDico($capitale,lc($capitale),Text2Phoneme(lc($capitale)),Cgram("VIL"),'','',2);
		my $definition="Capitale ";
		my $article;
		if ( $nombre   eq  'p' )   {  $article = " des "; }
		elsif ( $genre  eq 'f'  )  { $article = " de la ";}
		else {$article=" du ";}
		InsereDefinition($capitale,"Capitale"+$article+$pays,"Dicorime");
		$compt++;
		$progress->update($compt);
	}
	close (FIC);
	my $td1= new Benchmark;
	my $td = timediff($td1, $td0);
	print "Pays ($compt):",timestr($td),"\n";
}

sub CreationVilleFrance {

my %REGION = ('01', 'Guadeloupe','02','Martinique','03',"Guyane",'04',"La Réunion",'11',"Île-de-France",'21',"Champagne-Ardenne",'22',"Picardie",'23',"Haute-Normandie",'24',"Centre",'25',"Basse-Normandie",'26',"Bourgogne",'31',"Nord-Pas-de-Calais",'41',"Lorraine",'42',"Alsace",'43',"Franche-Comté",'52',"Pays de la Loire",'53',"Bretagne",'54',"Poitou-Charentes",'72',"Aquitaine",'73',"Midi-Pyrénées",'74',"Limousin",'82',"Rhône-Alpes",'83',"Auvergne",'91',"Languedoc-Roussillon",'93',"Provence-Alpes-Côte d'Azur",'94',"Corse" ) ;
my %REGION_GENRE = ('01', 'f','02','f','03',"f",'04',"f",'11',"f",'21',"f",'22',"f",'23',"f",'24',"f",'25',"f",'26',"f",'31',"m",'41',"f",'42',"f",'43',"f",'52',"m",'53',"f",'54',"m",'72',"f",'73',"m",'74',"m",'82',"m",'83',"f",'91',"f",'93',"f",'94',"f" ) ;

my @ChefLieu=("","chef lieu de canton ","chef-lieu d'arrondissement ","chef-lieu de département ","chef-lieu de région "),
my %DEPARTEMENT=(
"01","Ain",
"02","Aisne",
"03","Allier",
"04","Alpes-de-Haute-Provence",
"05","Hautes-Alpes",
"06","Alpes-Maritimes",
"07","Ardèche",
"08","Ardennes",
"09","Ariège",
"10","Aube",
"11","Aude",
"12","Aveyron",
"13","Bouches-du-Rhône",
"14","Calvados",
"15","Cantal",
"16","Charente",
"17","Charente-Maritime",
"18","Cher",
"19","Corrèze",
"2A","Corse-du-Sud",
"2B","Haute-Corse",
"21","Côte-d'Or",
"22","Côtes-d'Armor",
"23","Creuse",
"24","Dordogne",
"25","Doubs",
"26","Drôme",
"27","Eure",
"28","Eure-et-Loir",
"29","Finistère",
"30","Gard",
"31","Haute-Garonne",
"32","Gers",
"33","Gironde",
"34","Hérault",
"35","Ille-et-Vilaine",
"36","Indre",
"37","Indre-et-Loire",
"38","Isère",
"39","Jura",
"40","Landes",
"41","Loir-et-Cher",
"42","Loire",
"43","Haute-Loire",
"44","Loire-Atlantique",
"45","Loiret",
"46","Lot",
"47","Lot-et-Garonne",
"48","Lozère",
"49","Maine-et-Loire",
"50","Manche",
"51","Marne",
"52","Haute-Marne",
"53","Mayenne",
"54","Meurthe-et-Moselle",
"55","Meuse",
"56","Morbihan",
"57","Moselle",
"58","Nièvre",
"59","Nord",
"60","Oise",
"61","Orne",
"62","Pas-de-Calais",
"63","Puy-de-Dôme",
"64","Pyrénées-Atlantiques",
"65","Hautes-Pyrénées",
"66","Pyrénées-Orientales",
"67","Bas-Rhin",
"68","Haut-Rhin",
"69","Rhône",
"70","Haute-Saône",
"71","Saône-et-Loire",
"72","Sarthe",
"73","Savoie",
"74","Haute-Savoie",
"76","Seine-Maritime",
"77","Seine-et-Marne",
"78","Yvelines",
"79","Deux-Sèvres",
"80","Somme",
"81","Tarn",
"82","Tarn-et-Garonne",
"83","Var",
"84","Vaucluse",
"85","Vendée",
"86","Vienne",
"87","Haute-Vienne",
"88","Vosges",
"89","Yonne",
"90","Territoire de Belfort",
"91","Essonne",
"92","Hauts-de-Seine",
"93","Seine-Saint-Denis",
"94","Val-de-Marne",
"95",'Val-d\'Oise',
"971","Guadeloupe",
"972","Martinique",
"973","Guyane",
"974","La Réunion"
);

my %DEPARTEMENT_GENRE=(
"01","m",
"02","m",
"03","m",
"04","f",
"05","f",
"06","f",
"07","f",
"08","f",
"09","f",
"10","f",
"11","f",
"12","m",
"13","f",
"14","m",
"15","m",
"16","f",
"17","f",
"18","m",
"19","f",
"2A","f",
"2B","f",
"21","f",
"22","f",
"23","f",
"24","f",
"25","m",
"26","f",
"27","f",
"28","f",
"29","m",
"30","m",
"31","f",
"32","m",
"33","f",
"34","m",
"35","f",
"36","f",
"37","d",
"38","d",
"39","m",
"40","f",
"41","m",
"42","f",
"43","f",
"44","f",
"45","m",
"46","m",
"47","m",
"48","f",
"49","m",
"50","f",
"51","f",
"52","f",
"53","f",
"54","f",
"55","f",
"56","m",
"57","f",
"58","f",
"59","m",
"60","f",
"61","f",
"62","m",
"63","m",
"64","f",
"65","f",
"66","f",
"67","m",
"68","m",
"69","m",
"70","f",
"71","f",
"72","f",
"73","f",
"74","f",
"75","f",
"76","f",
"77","f",
"78","f",
"79","f",
"80","f",
"81","m",
"82","m",
"83","m",
"84","m",
"85","f",
"86","f",
"87","f",
"88","f",
"89","f",
"90","m",
"91","f",
"92","m",
"93","f",
"94","m",
"95","m",
"971","f",
"972","f",
"973","f",
"974","f"
);

my %DEPARTEMENT_NOMBRE=(
"01","s",
"02","s",
"03","s",
"04","p",
"05","p",
"06","p",
"07","s",
"08","p",
"09","s",
"10","s",
"11","s",
"12","s",
"13","p",
"14","s",
"15","s",
"16","s",
"17","s",
"18","s",
"19","s",
"2A","s",
"2B","s",
"21","s",
"22","p",
"23","s",
"24","s",
"25","s",
"26","s",
"27","s",
"28","s",
"29","s",
"30","s",
"31","s",
"32","s",
"33","s",
"34","s",
"35","s",
"36","s",
"37","s",
"38","s",
"39","s",
"40","p",
"41","s",
"42","s",
"43","s",
"44","s",
"45","s",
"46","s",
"47","s",
"48","s",
"49","s",
"50","s",
"51","s",
"52","s",
"53","s",
"54","s",
"55","s",
"56","s",
"57","s",
"58","s",
"59","s",
"60","s",
"61","s",
"62","s",
"63","s",
"64","p",
"65","p",
"66","p",
"67","s",
"68","s",
"69","s",
"70","s",
"71","s",
"72","s",
"73","se",
"74","s",
"75","s",
"76","s",
"77","s",
"78","p",
"79","s",
"80","p",
"81","p",
"82","s",
"83","s",
"84","s",
"85","s",
"86","s",
"87","S",
"88","p",
"89","s",
"90","s",
"91","S",
"92","p",
"93","s",
"94","s",
"95","s",
"971","s",
"972","s",
"973","s",
"974","s"
);


	 print ("Inclusion des villes de France\n");

	my  $td0 = new Benchmark;
	my $compt=0;
	my $progress = Term::ProgressBar->new($NBVilleFrance);
	open ( FIC, "$SourceDir/$FileVilleFrance");
	my $ligne= <FIC> ;
	while ( $ligne=<FIC> ) 
	{
		chomp ($ligne);
		$ligne=~s/\x0d//g;
		my @F=split(/\t/,$ligne);
		my $cheflieu=$F[1] ;
		my $nom=latin1($F[11])->utf8;
		my $region=$F[2] ;
		my $departement=$F[3] ;
		$region=$REGION{$region};
		my $dep=$DEPARTEMENT{$departement};
		my $definition ="Commune ".$ChefLieu[$cheflieu]."dans le département $departement ($dep) en région $region (France)";
		my $lowercase=lc($nom);
		InsereDico($nom,$lowercase,Text2Phoneme($lowercase),Cgram("VIL"),'m','s',2);
		InsereDefinition($nom,$definition,"INSEE");
		$compt++;
		$progress->update($compt);
	}
	foreach my $key (keys (%DEPARTEMENT) )
	{
		my $departement=$DEPARTEMENT{$key};
		my $lowercase=lc($departement);
		InsereDico($departement,$lowercase,Text2Phoneme($lowercase),Cgram("DEP"),$DEPARTEMENT_GENRE{$key},$DEPARTEMENT_NOMBRE{$key},2);
		my $definition="Département français ($key)";
		InsereDefinition($departement,$definition,"Dicorime");
	  }

	close(FIC);
	my $td1= new Benchmark;
	my $td = timediff($td1, $td0);
	print "Villes France ($compt):",timestr($td),"\n";
}



sub CreationVilleBelgique {
	print ("Inclusion des villes belges\n");
	my  $td0 = new Benchmark;
	my $compt=0;
	my $progress = Term::ProgressBar->new($NBVilleBelge);
	open ( FIC, "$SourceDir/$FileVilleBelgique");
	while ( my $ligne=<FIC> ) 
	{
		chomp ($ligne);
		$ligne=~s/\x0d//g;
		$compt++;
		my $definition ="Commune de Belgique";
		my $lowercase=lc($ligne);
		InsereDico($ligne,$lowercase,Text2Phoneme($lowercase),Cgram("VIL"),'m','s',2);
		$compt++;
		$progress->update($compt);
		$definition="Ville située en Belgique";
		InsereDefinition($ligne,$definition,"Dicorime");
	}
	close (FIC);
	my $td1= new Benchmark;
	my $td = timediff($td1, $td0);
	print "Villes Belges ($compt):",timestr($td),"\n";
}


sub CreationPrenoms {
	print ("Inclusion des prénoms\n");
my %TR;
$TR{"literature"}="littérature";
$TR{"italian"}="italien";
$TR{"maori"}="maori";
$TR{"new world mythology"}="monde nouvelle mythologie";
$TR{"medieval english"}="l'anglais médiéval";
$TR{"celtic mythology"}="la mythologie celtique";
$TR{"irish mythology"}="mythologie celtique irlandaise";
$TR{"roman mythology"}="la mythologie romaine";
$TR{"biblical (original)"}="biblique (original)";
$TR{"norwegian"}="norvégien";
$TR{"basque"}="basque";
$TR{"swedish"}="suédois";
$TR{"judeo-christian legend"}="légende judéo-chrétienne";
$TR{"theology"}="théologie";
$TR{"greek mythology (anglicized)"}="la mythologie grecque (anglicisé)";
$TR{"slavic mythology"}="mythologie slave";
$TR{"turkish"}="turc";
$TR{"frisian"}="frison";
$TR{"ancient egyptian"}="égyptien ancien";
$TR{"native american"}="native american";
$TR{"slovak"}="slovaque";
$TR{"egyptian mythology (hellenized)"}="mythologie égyptienne (hellénisé)";
$TR{"armenian"}="arménien";
$TR{"history"}="histoire";
$TR{"manx"}="manx";
$TR{"hindu mythology"}="mythologie hindoue";
$TR{"macedonian"}="macédonien";
$TR{"ancient greek"}="grec ancien";
$TR{"astronomy"}="astronomie";
$TR{"near eastern mythology (hellenized)"}="près de la mythologie orientale (hellénisé)";
$TR{"jewish"}="jewish";
$TR{"mythology"}="mythologie";
$TR{"vietnamese"}="vietnamien";
$TR{"greek"}="grec";
$TR{"mormon"}="mormon";
$TR{"estonian"}="estonien";
$TR{"bulgarian"}="bulgare";
$TR{"ancient greek (anglicized)"}="grec ancien (anglicisé)";
$TR{"latvian"}="letton";
$TR{"provençal"}="provençal";
$TR{"greek mythology (latinized)"}="la mythologie grecque (latinisé)";
$TR{"iranian"}="iranian";
$TR{"english (modern)"}="anglais (moderne)";
$TR{"english"}="anglais";
$TR{"indian"}="indian";
$TR{"biblical"}="bibliques";
$TR{"french"}="français";
$TR{"portuguese"}="portugais";
$TR{"serbian"}="serbe";
$TR{"germanic mythology"}="la mythologie germanique";
$TR{"romanian"}="roumain";
$TR{"ancient celtic"}="celtes";
$TR{"scottish"}="scottish";
$TR{"japanese"}="japonais";
$TR{"hungarian"}="hongrois";
$TR{"polish"}="polonais";
$TR{"anglo-saxon mythology"}="anglo-saxonne mythologie";
$TR{"anglo-saxon"}="anglo-saxonne";
$TR{"catalan"}="catalan";
$TR{"chinese"}="chinois";
$TR{"breton"}="breton";
$TR{"slovene"}="slovène";
$TR{"russian"}="russe";
$TR{"greek mythology"}="mythologie grecque";
$TR{"?"}="origine incertaine";
$TR{"anglo-saxon (latinized)"}="anglo-saxons (latinisé)";
$TR{"egyptian mythology"}="mythologie égyptienne";
$TR{"celtic mythology (latinized)"}="la mythologie celtique (latinisé)";
$TR{"khmer"}="khmer";
$TR{"galician"}="galicien";
$TR{"esperanto"}="espéranto";
$TR{"icelandic"}="islandais";
$TR{"thai"}="thai";
$TR{"finnish"}="finnois";
$TR{"korean"}="coréen";
$TR{"ancient celtic (latinized)"}="celtes (latinisé)";
$TR{"ancient greek (latinized)"}="grec ancien (latinisé)";
$TR{"czech"}="tchèque";
$TR{"african"}="african";
$TR{"ancient germanic"}="germanique ancien";
$TR{"spanish"}="espagnol";
$TR{"hawaiian"}="hawaïen";
$TR{"ukrainian"}="ukrainien";
$TR{"ancient germanic (latinized)"}="germanique ancien (latinisé)";
$TR{"norse mythology"}="mythologie nordique";
$TR{"dutch"}="néerlandais";
$TR{"german"}="allemand";
$TR{"danish"}="danois";
$TR{"scandinavian"}="scandinaves";
$TR{"lithuanian"}="lituanien";
$TR{"welsh"}="gallois";
$TR{"biblical (variant)"}="biblique (variante)";
$TR{"far eastern mythology"}="far eastern mythologie";
$TR{"cornish"}="cornique";
$TR{"ancient roman"}="rome antique";
$TR{"late roman"}="romain tardif";
$TR{"albanian"}="albanais";
$TR{"croatian"}="croate";
$TR{"arabic"}="arabe";
$TR{"ancient scandinavian"}="anciens scandinaves";
$TR{"irish"}="irlandais";
$TR{"near eastern mythology"}="près de la mythologie orientale";
$TR{"welsh mythology"}="gallois mythologie ";


	my  $td0 = new Benchmark;
	my $compt=0;
	my $progress = Term::ProgressBar->new($NBPrenom);
	open ( FIC, "<$SourceDir/$PrenomsFile");
	my $ligne= <FIC> ;


	while ( $ligne=<FIC> ) {
		chomp ($ligne);
		$ligne=~s/\x0d//g;
		my @F=split(/\t/,$ligne);
		my $ortho=latin1($F[0])->utf8;
		my $lemme=lc($ortho);
		$ortho=~s/\(\d\)//;
		$ortho=~s/^'//;
		$ortho=~s/õ/o/g;
		$ortho=~s/á/a/g;
		$ortho=~s/æ/e/g;
		$ortho=~s/ð/o/g;
		$ortho=~s/å/a/g;
		$ortho=~s/ó/o/g;
		$ortho=~s/ú/u/g;
		$ortho=~s/í/i/g;
		$ortho=~s/ì/i/g;
		$ortho=~s/þ/b/g;
		$ortho=~s/ã/a/g;
		my @compose=split(/-/,$ortho);
		foreach my $m (@compose) { $m=ucfirst($m);}
		$ortho=join('-',@compose);
		my $genre=latin1($F[1])->utf8;
		my $freq=latin1($F[3])->utf8;
		my $langue=latin1($F[2])->utf8;
		InsereDico($ortho,$lemme,Text2Phoneme(lc($ortho)),Cgram("PRN"),$genre,'s',$freq);
		$compt++;
		my $definition;
		$progress->update($compt);

		my @langues=split(/,/,$langue);
		my @tradlangues;
		foreach my $l ( @langues) { $l=~s/^ +//g;  $l=~s/ $//g; push(@tradlangues,$TR{$l}); }
		my $stradl=join(",",@tradlangues);
		if ( $genre eq "f" ) {$definition ="Prénom féminin\n"; }
		elsif ( $genre eq "m" ) { $definition = "Prénom masculin\n" ;}
		elsif ( $genre eq "m,f"  or $genre eq "f,m" ) { $definition="Prénom masculin et féminin\n";}
		$definition.="\nOrigines: $stradl";
		InsereDefinition($ortho,$definition,"Lexique");
		}
		close (FIC);
		my $td1= new Benchmark;
			 my $td = timediff($td1, $td0);
			 print "Prenoms ($compt) :",timestr($td),"\n";
}


sub CreationRimes  {
	print "Création de la table de rimes\n";
	my  $td0 = new Benchmark;
	my $progress = Term::ProgressBar->new($NBLexique);
	my $compt=0;

	open ( FIC, "$SourceDir/$LexiqueFile");
	my $ligne= <FIC> ;


	while ( $ligne=<FIC> ) {
		chomp ($ligne);
		my @F=split(/\t/,$ligne);
		my @ph=split(//, $F[1]);
		my $ph1=latin1(pop(@ph))->utf8;
		my $ph2=latin1(pop(@ph))->utf8;
		my $ph3=latin1(pop(@ph))->utf8;
		my $ph4=latin1(pop(@ph))->utf8;
		my $ph5=latin1(pop(@ph))->utf8;
		my $ph6=latin1(pop(@ph))->utf8;
		my $ph7=latin1(pop(@ph))->utf8;
		my $ph8=latin1(pop(@ph))->utf8;
		my $ortho=$F[0];
		my $lemme=$F[2];
		my $cgram=Cgram($F[3]);
		my $genre=$F[4];
		my $nombre=$F[5];
		my $nbsyll=$F[23];
		my $freq=$F[9];
		my $PH=latin1($F[1])->utf8;
		$compt++;
		if ( ($cgram eq "VER") and ( $ortho ne $lemme ) ){ ;}
		else{
		$db->do("INSERT  INTO dico ( ortho, lemme, cgram, genre, nombre, nbsyll, freq, PH,PH1,PH2,PH3,PH4,PH5,PH6,PH7,PH8 ) VALUES ( \"$ortho\", \"$lemme\", \"$cgram\", \"$genre\",\"$nombre\", \"$nbsyll\", \"$freq\", \"$PH\" ,\"$ph1\",\"$ph2\",\"$ph3\",\"$ph4\",\"$ph5\",\"$ph6\",\"$ph7\",\"$ph8\" )") ;
		}
		 $progress->update($compt);
		}
	my $td1 = new Benchmark;
	my $td = timediff($td1, $td0);
	print "Rime ($compt) :",timestr($td),"\n";
}

sub CreationCitations {
	print "Création de la table de citations\n";
	my  $td0 = new Benchmark;
	my $progress = Term::ProgressBar->new($NBCitation);
	my $compt=0;
	opendir  DIR1, "$SourceDir/$FortuneURL";
	my $citation="";
	my $auteur="";
	my $titre; my $reference; my $requete;
	my $requete0="INSERT  INTO citations (genre,rubrique,citation,auteur) VALUES (";

	while ( my $genre = readdir (DIR1) ){
		if (  $genre !~ /^\./  and $genre !~ /^Makefile/ ) {
			opendir DIR2,"$SourceDir/$FortuneURL/$genre" ;
			while ( my $rubrique = readdir (DIR2) )	{
				if (  $rubrique !~ /^\./  and $rubrique !~ /^Makefile/ ) {
				open FILE, "$SourceDir/$FortuneURL/$genre/$rubrique";
				while ( my $ligne = <FILE> ) {
					$ligne=latin1($ligne)->utf8;
					if ( $ligne =~/^%$/ ) {
					$citation=~s/"//g;
					$citation=~s/^\n//g;
					my $requete="INSERT  INTO citations (genre,rubrique,citation,auteur) VALUES ( \"".ArrangeGenre($genre)."\",\"".ArrangeRubrique($rubrique)."\",\"$citation\",\"$auteur\")";
					$db->do($requete);
					$citation="";
					$auteur="";
					$compt++;
					 $progress->update($compt);
					}
					elsif ( $ligne =~/-\+-/ ){
					my @tab=split(/-\+-/,$ligne);
					pop(@tab);
					$auteur=pop(@tab);
					$auteur=~s/"/''/g;
					}
					else  { chomp($ligne);
						$citation.= " $ligne" }
					}
				close (FILE);

				}
			}

		closedir (DIR2);
		}
	}
	closedir (DIR1);
	my $nb=0;
	my $nbcitation=0;
	my $nbref=0;

	open ( FILE,"<$SourceDir/$Wikiquotefile") ;
	$/="";

	$requete=$requete0;
	while ( my $ligne=<FILE> )
	{
			$nb++;
		$ligne=~s/&amp;nbsp;/ /g;
		$ligne=~s/\{\{Personnage *\|([^}]+)\}\}/$1/gi;
		if (  $ligne=~/^\{\{Citation([^}]*)\}\}/si ) {
			$citation=$1;
			$citation=~s/\|citation=//i;
			$citation=~s/\|//;
			$nbcitation++;
			$citation=~s/\|.*//g;
			$citation=~s/original *=.*//ig;
			$citation=ArrangeWikiQuote($citation);
			}
		if ( $ligne=~/\{\{Réf ([^|]*)\|titre=([^|]*)\|auteur=([^|]*)/i )  {
			$nbref++;
			$reference=$1;
			$titre=$2;
			$auteur=$3;
		}
		if ( $ligne=~/\{\{Réf/i ) {
			$titre=ArrangeWikiQuote($titre);
			$reference=ArrangeWikiQuote($reference);
			$auteur=ArrangeWikiQuote($auteur);
			$auteur=~s/\[//g;
			$auteur=~s/\]//g;
			if ( $citation ne "" ) {
			$requete=$requete0."\"$reference\",\"$titre\",\"$citation\",\"$auteur\")";
			$db->do($requete) || print "ERROR * * * * :[$requete]\n";	
			}
			$compt++;
			$progress->update($compt);
		}
		if ( $ligne=~/<title>(.*)<\/title>/i ) {
			$reference="";
			$auteur="";
			$titre=$1;
			$citation="";
			$requete=$requete0;
		}
		$ligne=~s/\[\[w://g;
		$ligne=~s/\[\[//g;
		$ligne=~s/\]\]//g;
		$ligne=~s/langue *= *fr//g;
		$auteur=~s/:*w://g;
	}
	close (FILE);
	$/="\n";
	my $td1= new Benchmark;
	my $td = timediff($td1, $td0);
	print "Citations ($compt) :",timestr($td),"\n";
}

sub CreationSynonymes {
print "Création de la table des synonymes\n";
	my  $td0 = new Benchmark;
	my $progress = Term::ProgressBar->new($NBSynonymes);
	my $compt=0;
	open  (FILE, "$SourceDir/$DictSyn") or die ("pas de fichier $SourceDir/$DictSyn");
	my $ligne=<FILE>;
	my $ortho;
	my $cgram;
	my $liste;
	while ($ligne)
	{
		$ligne=<FILE>;
		$compt++;
		my ($mot,$nb)=split(/\|/,$ligne);
		chomp($nb);
		$ortho=$mot;
		my $boucle=0;
		while ($boucle!=$nb)
		{
	 $boucle++;
	 $ligne=<FILE>;
	 $compt++;
	 $progress->update($compt);
	 chomp($ligne);
	 my @liste=split(/\|/,$ligne);
	 $cgram=shift(@liste);
	 $cgram=~s/\(//g;$cgram=~s/\)//g;
	 $cgram=~s/nom fem/Nom/;
	 $cgram=~s/participe présent/Adverbe/;
	 $cgram=~s/Adjectif Adjectif cardinal/Adjectif/;
	 $cgram=~s/Adjectif cardinal/Adjectif/;
	 $cgram=~s/Pronom personnel Pronom/Pronom/;
	 $cgram=~s/Pronom personnel/Pronom/;
	 $cgram=~s/adj epi/Adjectif/;

	 my @cgram=split(" ",$cgram);

	 foreach my $cgr (@cgram)
	 {
		 $cgram=convertitcgram($cgr,$ortho);
		 $liste=join("|",@liste);
		 my $requete="INSERT  INTO synonymes (ortho,cgram,liste) VALUES (\"$ortho\",\"$cgram\",\"$liste\")";
		 $db->do($requete);

	 }
		}
	}
	close (FILE);
	my $td1= new Benchmark;
	my $td = timediff($td1, $td0);
	print "Synonymes ($compt) :",timestr($td),"\n";
}

sub FormateDefinitions {
	my $Test= shift;
	print "Formatage de la table des définitions\n";
	my $progress;
	if ( ! $Test ) { $progress = Term::ProgressBar->new($NBWiktionary); }
	my $compt=0;
	my $td0= new Benchmark;
	my $source = "$SourceDir/$WiktionaryFile";
	if ( $Test ) { $source.=".test" }
	open (FIC ,"<$source");
	my @buf;
	$compteurdefinition=0 ;
	my $mot="";
	my $balisetexte=0;
	#On balaie l'ensemble des lignes
	while ( my $ligne = <FIC> )
	{
	if ( $ligne =~/<title>(.*)<\/title>/ ) { # marqueur de titre
	my $title=$1;
	#on est sur un titre donc on peut traiter le précédent qui est rangé dans buffer
	if ( $mot =~/^[a-zA-ZàÀâÂäÄéÉèÈëËêÊïÏîÎöÖôÔùÙûÛüÜ]+$/ ) {
		#le mot précédent qui sert de marqueur ne comprend pas de blanc
		# ne contient pas de caractères non français
		my $l=join ( '',@buf);
		# il faut aussi que le buffer contiennent =fr= sinon c'est pas un mot français
			if ( $l =~/== *\{\{langue\|fr}} *==/ ) { InsereDefinition($mot,$l,"Wiktionary"); }
		}
		 $mot=$title;
		 $balisetexte=0;
	}
	elsif ( $ligne =~/<text[^>]*>(.*)/ ) {
		 @buf=();
		 push ( @buf,$1);
		 $balisetexte=1;
	}
	elsif ( $ligne=~m;(.*)</text>; ){
		 push ( @buf,$1);
		 $balisetexte=0;
	}
	elsif ( $balisetexte==1 ) {
		 push ( @buf,$ligne);
	}
	$compt++;
	$progress->update($compt);

  }
   my $td1= new Benchmark;
	my $td = timediff($td1, $td0);
	print "Formate définitions ($compteurdefinition) :",timestr($td),"\n";
	$db->commit;
	$db->do("DELETE FROM definitions WHERE definitions.ortho NOT IN ( SELECT dico.ortho FROM dico )");
	$db->do("DELETE  FROM dico  WHERE dico.ortho NOT IN ( SELECT definitions.ortho FROM definitions )");
	$db->commit;
	$db->{AutoCommit} = 1;
	$db->do("VACUUM");	
	$db->{AutoCommit} = 0;
}

sub CreationMotsRares()
{
	 my $xml = new XML::Simple;
	 my $doc = $xml->XMLin("$SourceDir/$DicMotsRare");
	 print ("Inclusion des mots rares du dictionnaire de Méral\n");
	 my  $td0 = new Benchmark;
	 my $compt=0;
	 my $progress = Term::ProgressBar->new($NBMotsRares);
	 $compteurdefinition=0;
	 $db->commit;
	 my %MASCULINFEMININ;
	 $MASCULINFEMININ{'IE'}="I";
	 $MASCULINFEMININ{'USE'}="US";
	 $MASCULINFEMININ{'ENTE'}="ENT";
	 $MASCULINFEMININ{'ÉE'}="É";
	 $MASCULINFEMININ{'INE'}="IN";
	 $MASCULINFEMININ{'ITE'}="IT";
	 $MASCULINFEMININ{'IENNE'}="IEN";
	 $MASCULINFEMININ{'ONGUE'}="ONG";
	 $MASCULINFEMININ{'ONNE'}="ON";
	 $MASCULINFEMININ{'OTTE'}="OT";
	 $MASCULINFEMININ{'ENNE'}="EN";
	 $MASCULINFEMININ{'ARDE'}="ARD";
	 $MASCULINFEMININ{'OTTE'}="OT";
	 $MASCULINFEMININ{'EUSE'}="EUX";
	 $MASCULINFEMININ{'ELLE'}="EL";
	 $MASCULINFEMININ{'EOISE'}="EOIS";
	 $MASCULINFEMININ{'IVE'}="IF";
	 $MASCULINFEMININ{'UE'}="U";
	 $MASCULINFEMININ{'TRICE'}="TEUR";
	 $MASCULINFEMININ{'ANTE'}="ANT";
	 $MASCULINFEMININ{'AISE'}="AIS";
	 $MASCULINFEMININ{'OSE'}="OS";
	 $MASCULINFEMININ{'ARSE'}="ARS";
	 $MASCULINFEMININ{'ÈRE'}="ER";
	 $MASCULINFEMININ{'ALE'}="AL";
	 $MASCULINFEMININ{'ETTE'}="ET";
	 $MASCULINFEMININ{'ATE'}="AT";
	 $MASCULINFEMININ{'VIVE'}="VIF";
	 $MASCULINFEMININ{'IÈRE'}="IER";
	 $MASCULINFEMININ{'ISE'}="IS";
	 $MASCULINFEMININ{'OISE'}="OIS";
	 $MASCULINFEMININ{'EULE'}="EUL";
	 $MASCULINFEMININ{'EILLE'}="EIL";
	 $MASCULINFEMININ{'ERSE'}="ERS";
	 $MASCULINFEMININ{'AUDE'}="AUD";
	 $MASCULINFEMININ{'NÉE'}="NE";
	 $MASCULINFEMININ{'ANNE'}="AN";
	 $MASCULINFEMININ{'IDE'}="ID";
	 $MASCULINFEMININ{'ORSE'}="ORS";
	my $forme;
	my $genre;
	my $enrichi;
	foreach my $lettre ( @{$doc->{sect1}})
	{
	  foreach my $ligne ( @{$lettre->{para}} )
	  {
	  	my $feminin;
	  	### Astuce pour pas prendre les lignes vides
		  if ( $ligne == "" ) 
		  {
			my ( $mot,@def)=split(/:/,$ligne);
			my $def=join(':',@def);
			  $mot=~s/\s+/ /g;
			  $mot=~s/ $//g;
			  $def=~s/\s+/ /g;
			  $def=~s/^ //g;
			  
			  #La forme féminine éventuelle est donnée en MAJUSCULE
			  if ( $def=~/^(\p{IsUpper}+) / )
			   {
					my $termfem=$1;
					my $termfemminus=lc($termfem);
					my $termmasculminus=lc($MASCULINFEMININ{$termfem});
					$feminin=$mot;
					$feminin=~s/$termmasculminus$/$termfemminus/;
					$def=~s/$termfem //;	
				}
				$def=~s/^\([^)]+\)//g;
				if ( $def=~/^, *([a-zA]+)\./ )
				{
					$forme=$1;
					if ( $forme eq "n" ) { $forme = "NOM" ; $def=~s/^, *n\.//;}
					elsif ( $forme eq "adj" ) { $forme ="ADJ" ;$def=~s/^, *adj\.//;}
					elsif ( $forme eq "prép" ) { $forme ="PRE" ;$def=~s/^, *prép\.//;}
					elsif ( $forme eq "v" ) { $forme ="VER" ;$def=~s/^, *v\.//;}
					elsif ( $forme eq "adv" ) { $forme ="ADV" ;$def=~s/^, *adv\.//;}
					elsif ( $forme eq "adverbe" ) { $forme ="ADV" ;$def=~s/^, *adverbe\.//;}
					elsif ( $forme eq "conj" ) { $forme ="CONJ" ;$def=~s/^, *conj\.//;}
					elsif ( $forme eq "interj" ) { $forme ="ONO" ;$def=~s/^, *interj\.//;}	
					}
				if ( $def=~/^ *(.)\./ ) 
				{
               $genre=$1;
               $def =~s/^ *.\.//;
            }
				#$def=~s/^, *//;
				my $motminus=lc($mot);
				if ( $mot!~/ /)
				{
					$compt++;
					if ( ! LemmePresentDefinition($motminus) ) 
						{  $def=".MERAL.$def";
                     InsereDefinition($motminus,$def,"MERAL");
                  }
					if ( $feminin && ! MotPresentDico($feminin) ) 
					{  
                  if ( ! MotPresentDico($motminus) ) 
                  {  InsereDico($motminus,$motminus,Text2Phoneme($motminus),Cgram($forme),'m','s',2);}
						InsereDico($feminin,$motminus,Text2Phoneme($feminin),Cgram($forme),'f','s',2); 
					}
					elsif ( ! MotPresentDico($motminus) )
						{InsereDico($motminus,$motminus,Text2Phoneme($motminus),Cgram($forme),$genre,'s',2);}
					$progress->update($compt);
				}
			}
		}
	}
	close (FIC);
	 my $td1= new Benchmark;
	 my $td = timediff($td1, $td0);
	 print "Mots rares ($compt):",timestr($td),"\n";
	 $db->commit;
}


sub InsereDefinition()
{
	 my $mot=shift;
	 my $definition=shift;
	 my $origin=shift;
	 
	 $compteurdefinition++;
	 if (  Sigle($mot) ne ""  ) #Traitement des sigles
	{
				  my $sigle=Sigle($mot);
				  InsereDico($sigle,$sigle,Text2Phoneme(Sigle($sigle)),Cgram("SIG"),'','',2);
				  $db->do("INSERT INTO definitions (ortho,definition,origin) VALUES (\"$sigle\",".$db->quote($definition).",\"SIGLE\")");
	}
	else 
	{  
		my $requete="INSERT INTO definitions (ortho,definition,origin) VALUES (\"$mot\",".$db->quote($definition).",\"$origin\")";
		if ( $mot eq "niaque" || $mot eq "façon" || $mot eq "Lescar" ) 
		{
			trace ( $requete,$definition,$mot);
		}
		$db->do($requete) || print "Error : $requete\n";

	}
	 if ( $compteurdefinition % 10000 == 0 )
	 {
	 $db->commit;
	 controlsizetable();
	 }

}

sub trace
{
	my $requete=shift;
	my $definition=shift;
	my $ortho=shift;
	open (LOG,">>dico.log");
	print LOG "* * * * * *\nORTHO:$ortho\n";
	print LOG "REQUETE:$requete\n";
	print LOG "REQUETE:$definition\n";
	close LOG;
}

sub controlsizetable
{
	my $req1="SELECT count(*) FROM dico";
	my $req2="SELECT count(*) FROM definitions";
	
	my $sth1 = $db->prepare($req1);
	$sth1->execute();
	my @count1 = $sth1->fetchrow();
	my $sth2 = $db->prepare($req2);
	$sth2->execute();
	my @count2 = $sth2->fetchrow();
	
	print "Compteurs => dico: @count1 definitions : @count2\n";
	
}

sub CreationVerbiste
{
   my $lastverbe;
   my %VAL=
   ( 
   "indicativepresent"=>"A",
   "indicativeimperfect"=>"B",
   "indicativefuture"=>"C",
   "indicativepast"=>"D",
   "conditionalpresent"=>"E",
   "subjunctivepresent"=>"F",
   "subjunctiveimperfect"=>"G",
   "imperativepresent"=>"H",
   "participlepresent"=>"I",
   "participlepast"=>"K"
   );
   print "Formatage des conjugaisons\n";
   my $progress;
   $progress = Term::ProgressBar->new($NBVERBES);
   my $compt=0;
   my $td0= new Benchmark;
   
   my @verbes=`french-conjugator --all-infinitives`;
   
   my $value="";
   my $personne=1;
   foreach  my $verbe (@verbes)
   {
      $lastverbe=$verbe;
      $compt++;
      my $nbv=0;
      chomp($verbe);
      if ( $verbe=~/'/ ) { $verbe=~s/'/\\'/ ; }
      my $command="french-conjugator $verbe";
      my @result=`$command`;
      my %V=();
      foreach my $res (@result)
      {
         chomp($res);
         if ( $res=~/^-([a-z ]+)/ ) { my $r=$1;$r=~s/://;$r=~s/ //g;  $value=$VAL{$r} ;$personne=1;}
         elsif ( $res!~/^-$/  && $res !~/^$/ ) 
         {
            if ( $value )  
            {
            # impératif on augmente la personne
            if ( $value eq "H" and $personne eq "1" ) { $personne ++ ; }
            elsif ($value eq "H" and ( $personne eq "2" or $personne eq "1" ) ) { $personne+=2; }
            $V{$res}.= "${personne}${value}";$personne++;}
         }
      }
      foreach my $v (keys(%V) )
      {
         $nbv++;
         my $PH=Text2Phoneme($v);
         $PH=BugPreset($PH);
         my @ph=split(//,$PH);
         $PH=BugSet($PH);
         my $ph1=BugSet(pop(@ph));
         my $ph2=BugSet(pop(@ph));
         my $ph3=BugSet(pop(@ph));
         my $ph4=BugSet(pop(@ph));
         my $ph5=BugSet(pop(@ph));
         my $ph6=BugSet(pop(@ph));
         my $ph7=BugSet(pop(@ph));
         my $ph8=BugSet(pop(@ph));
         my $nbsyll=NbSyllabe($PH);
         my $pluriel=$V{$v};
         my $nombre="";
         $pluriel=~s/[a-z]+//g;
         my $singulier=$pluriel;
         $pluriel=~s/[456]/p/g;
         $singulier=~s/[123]/s/g;
         if ( $singulier !~/p/ and $singulier =~/s/ ) { $nombre="s";}
         elsif ( $pluriel !~/s/ and $pluriel =~/p/ ) { $nombre="p";}
         # Suppression des alternatives 
         my @listv=split(/,/,$v);
         my $prem=shift (@listv);
         my $requete="INSERT  INTO dico ( ortho, lemme, cgram,nombre, genre, nbsyll, freq, PH,PH1,PH2,PH3,PH4,PH5,PH6,PH7,PH8 ) VALUES ( \"$prem\", \"$verbe\", \"VER\",\"$nombre\", \"".$V{$v}."\", \"$nbsyll\",\"\", \"$PH\" ,\"$ph1\",\"$ph2\",\"$ph3\",\"$ph4\",\"$ph5\",\"$ph6\",\"$ph7\",\"$ph8\" )";
         $db->do($requete);
      }
      $progress->update($compt);
   }
	my $td1= new Benchmark;
	my $td = timediff($td1, $td0);
	print "Conjugaison ($compt):",timestr($td),"\n";
	print "Last verbe = $lastverbe\n";
}



sub CreateIndexes {
   print "Création des index\n";
my $td1 = new Benchmark ;
$db->do("CREATE INDEX indexortho on dico ( ortho )");

$db->do("CREATE INDEX indexph on dico ( PH )");
$db->do("CREATE INDEX indexph1 on dico ( PH1 )");
$db->do("CREATE INDEX indexph2 on dico ( PH2 )");
$db->do("CREATE INDEX indexph3 on dico ( PH3 )");
$db->do("CREATE INDEX indexph4 on dico ( PH4 )");
$db->do("CREATE INDEX indexph5 on dico ( PH5 )");
$db->do("CREATE INDEX indexph6 on dico ( PH6 )");
$db->do("CREATE INDEX indexph7 on dico ( PH7 )");
$db->do("CREATE INDEX indexph8 on dico ( PH8 )");
$db->do("CREATE INDEX indexph9 on dico ( PH1,PH2,PH3,PH4,PH5,PH6,PH7,PH8 )");
$db->do("CREATE INDEX indexcitation on citations ( citation )");
$db->do("CREATE INDEX indexortho2 on synonymes ( ortho )");
$db->do("CREATE INDEX  indexortho3   on definitions ( ortho )");
my $td2 = new Benchmark;
my $td = timediff($td1, $td2);
}


sub ArrangeRubrique {
my $rubrique=shift;
$rubrique=~s/_/ /g;
$rubrique=~s/"/''/g;

		my @noms=split(/ /,$rubrique) ; foreach ( @noms) { $_=ucfirst } ;
		$rubrique=join(' ',@noms);
		my @noms=split(/-/,$rubrique) ; foreach ( @noms) { $_=ucfirst } ;
		$rubrique=join(' ',@noms);

}

sub ArrangeGenre  {
my $genre=shift;
	$genre =~s/_/ /g;
	$genre =~s/francaise/française/g;
	$genre =~s/litterature/littérature/g;
	$genre =~s/personnalites/personnalités/g;
	$genre =~s/cinema/cinéma/g;
	$genre =~s/etrangere/étrangère/g;
	$genre =~s/bd/bande dessinée/g;
	$genre =~s/fr\.rec\.humour/humour/g;
	$genre =~s/fr\.rec\.photo/photo/g;
	$genre =~s/GDP/humour d'informaticien/g;
	$genre =~s/"/''/g;
	return ($genre);
}

sub SetMarqueur
{
	my $m=shift;
	ResetMarqueurs();
	$marqueur{$m}=1;
}

sub ResetMarqueurs
{
	foreach (@marqueurtab)
		{ $marqueur{$_} = 0 }
}

sub ArrangeWiktionary
{
my $utfl= shift ;
my $utfl=EpureGlobal($utfl);
# utf8::encode($utfl);
$utfl=~s/\[//g;
$utfl=~s/\]//g;
$utfl=~s/\'//g;
$utfl=~s/#//g;
$utfl=~s/://g;
$utfl=~s/\*//g;
$utfl=~s/\{\{\(\}\}//g;
$utfl=~s/\{\{\)\}\}//g;
$utfl=~s/\{\{-\}\}//g;
$utfl=~s/\{\{/(/g;
$utfl=~s/\}\}/)/g;
$utfl=~s/\([^)]*\)//g;
$utfl=~s/\n\n/\n/g;
$utfl=~s/^\n//g;
$utfl=~s/\n$//g;
$utfl=~s/\n/,/g;
$utfl=~s/.*\[\[(.*)\]\].*/$1/g;
$utfl=~s/'''.*//;



return $utfl;
}




sub ArrangeWiktionaryMotsComposes
{
	my $ligne=shift;
	$ligne=~s/\n/,/g;
	$ligne=~s/^,//g;
	$ligne=~s/,,//g;
	$ligne=~s/,$//g;
	if ($ligne ne "" ) { $ligne="Mots composés : $ligne";}
	return ($ligne);
}




sub Sigle
{
  my $mot=shift;
  if ( $mot=~/^[A-Z.]+$/ )
	{
		$mot=~s/\.//g;
	}
  else { $mot="";}
  if ( length($mot) <= 2 ) { $mot="";}
  return $mot;
}



sub EpureGlobal
{
my $utfl= shift ;
$utfl=~s/<[^>]+>//g;
$utfl=~s/\(-réf-\)//g;
$utfl=~s/\(R:DAF8\)//g;
$utfl=~s/&lt;[^&]*&gt;//g;			# suppression des &lt;small&gt;
return $utfl;
}

sub NombreLigne
{
 my $file=shift;
 print "Comptage $file";
 open ( FIC,$file) ;
 my $compt=0;
 my $ligne;
 while ( $ligne= <FIC> )  { $compt++ }
 close (FIC);
 print " [$compt]\n";
 return ($compt);
}

sub ArrangeWikiQuote
{
 my $phrase=shift;
# mise de quotes
 chomp($phrase);
 while ($phrase=~/&quot;/ ) {
	 if ( $quoteouvrante == 1 ) {
	$phrase=~s/&quot;/«/;
	$quoteouvrante=0;
	}
	 elsif ( $quoteouvrante == 0 ) {
	$phrase=~s/&quot;/»/;
	$quoteouvrante=1;
	}
	 }
 #Supression des balises restantes
 $phrase=~s/&lt;[^&]*\/{0,1}&gt;//g;
 #Suppression des &amp;
 $phrase=~s/\{\{tab//gi;
 $phrase=~s/&amp;/&/g;
 $phrase=~s/&lt;/</g;
 $phrase=~s/&gt;/>/g;
 $phrase=~s/^1=\n//g;
 $phrase=~s/^citation = //ig;
 $phrase=~s/&thinsp;/ /g;
 $phrase=~s/&mdash;/-/g;
 $phrase=~s/^\n+//g;
 return ($phrase);
}


sub BugPreset {
my $par=shift;
$par=~s/§/M/g;
$par=~s/°/P/g;
return ($par);
}

sub BugSet {
my $par=shift;
$par=~s/M/§/g;
$par=~s/P/°/g;
return ($par);
}

sub CheckFiles
{
  -f "$SourceDir/$LexiqueFile"  ||  Error ("Le fichier Lexique est manquant : $SourceDir/$LexiqueFile!");
  -f "$SourceDir/$PrenomsFile" || Error ("Le fichier Prénom est manquant $SourceDir/$PrenomsFile!");
  -f "$SourceDir/$FileVilleFrance" || Error ("Le fichier Villes de France est manquant $SourceDir/$FileVilleFrance!");
  -d "$SourceDir/$FortuneURL" || Error ("Le fichier Fortune est manquant $SourceDir/$FortuneURL!");
  -f "$SourceDir/$WiktionaryFile" || Error ("Le fichier Wiktionary est manquant $SourceDir/$WiktionaryFile!");
  -f "$SourceDir/$Wikiquotefile" || Error ("Le fichier Wikiquote est manquant $SourceDir/$Wikiquotefile!");
  print "Tous les fichiers semblent présents !\n "
}

sub CheckPrograms()
{
 my $prog=`which french-conjugator`;
 if ( $prog eq "" )
  {
	 print "French conjugator absent ! ( sudo apt-get install verbiste )\n";
	 exit ();
  }
}

sub Error
{
  my $message=shift;
  print "$message\n" ;
  print "Avez vous bien lancé getfiles.pl avec les bonnes valeurs des fichiers ?\n";
  print "Il faut vérifier sur les sites donnés en URL dans getfile que les fichiers sont aux bonnes versions\n";
  exit();
}

sub ArrangeBaseSQL
{
   print "Arrangement de certains champs de la base\n";
	$db->do("UPDATE citations SET auteur = rubrique  WHERE  auteur =''") ;
	$db->do("UPDATE citations  SET auteur=substr(auteur,4,1000 ) WHERE  auteur LIKE ':_:%'");
	#Remplacement des O de cul de poule par des o normaux
	$db->do("UPDATE dico SET  PH=replace(PH,'O','o')");
	$db->do("UPDATE dico SET  genre=replace(genre,'4H','5H')");
	$db->do("UPDATE dico SET  genre=replace(genre,'3H','4H')");

	$db->do("UPDATE dico set PH8='o' where PH8='O'");
	$db->do("UPDATE dico set PH7='o' where PH7='O'");
	$db->do("UPDATE dico set PH6='o' where PH6='O'");
	$db->do("UPDATE dico set PH5='o' where PH4='O'");
	$db->do("UPDATE dico set PH4='o' where PH4='O'");
	$db->do("UPDATE dico set PH3='o' where PH3='O'");
	$db->do("UPDATE dico set PH2='o' where PH2='O'");
	$db->do("UPDATE dico set PH1='o' where PH1='O'");
	$db->commit;
}


sub debug
{
 my $chaine=shift;
 if ( $Test ) { print "debug  $chaine\n";}
}


sub Cgram
{
 my $chaine = shift;
 my @retour;
 my @list=split(/,/,$chaine);
 foreach my $cgram (@list)
 {
 ## Issus de lexiques
 # Adjectif
 if ( $cgram=~/ADJ/i )  {push ( @retour,"ADJ");}
 # Adverbe
 if ( $cgram=~/ADV/i ) {push ( @retour,"ADV");}
 # Article
 if ( $cgram=~/ART/i ) {push ( @retour,"ADV");}
 # Verbe auxilliaire
 if ( $cgram=~/AUX/i ){ push ( @retour,"AUX");}
 # Conjonction de coordination 
 if ( $cgram=~/CON/i ){ push ( @retour,"CON");}
 # Nom
 if ( $cgram=~/NOM/i ){ push ( @retour,"NOM");}
 # Onomatopée
 if ( $cgram=~/ONO/i ){ push ( @retour,"ONO");}
 # Préposiion 
 if ( $cgram=~/PRE/i ){ push ( @retour,"PRE");}
 # Formes pronominales
 if ( $cgram=~/PRO/i ) {push ( @retour,"PRO");}
 # VERBE
 if ( $cgram=~/VER/ ) {push ( @retour,"VER");}
 # Ajouté par qdicorime
 # Villes
 if ( $cgram=~/VIL/i ) {push ( @retour,"VIL");}
 # Département
 if ( $cgram=~/DEP/i ) {push ( @retour,"DEP");}
 # Région
 if ( $cgram=~/REG/i ) {push ( @retour,"REG");}
 # Pays
 if ( $cgram=~/PAY/i ) {push ( @retour,"PAY");}
 #Prénom
  if ( $cgram=~/PRN/i ){ push ( @retour,"PRN");}
 #Sigle
 if ( $cgram=~/SIG/i ) {push ( @retour,"SIG");}
 }
 return ( join (',',@retour));
}


sub convertitcgram{
  my $cgram=shift;
  my $ortho=shift;
  if ( $cgram =~/Adjectif/i ) { return ("ADJ");}
  elsif ($cgram =~/Nom/i ) { return ("NOM");}
  elsif ($cgram =~/Preposition/i ) { return ("PRE");}
  elsif ($cgram =~/Verbe/i ) { return ("VER");}
  elsif ($cgram =~/Adverbe/i ) { return ("ADV");}
  elsif ($cgram =~/Interjection/i ) { return ("ONO");}
  elsif ($cgram =~/Determinant/i ) { return ("AUX");}
  elsif ($cgram =~/Pronom/i ) { return ("PRO");}
  elsif ($cgram =~/Conjonction/i ) { return ("CON");}
  elsif ($cgram =~/Abreviation/i ) { return ("SIG");}
  elsif ($cgram =~/Participe/i ) { return ("ADV");}
  elsif ($cgram eq "?" ) { return ("");}
  else { print "$cgram sur $ortho\n";exit}
}

sub InsereDico
{
	my $ortho=shift;
	my $lemme=shift;
	my $phonemisation=shift;
	my $cgram=shift;
	my $nb=shift;
	my $genre=shift;
	my $freq=shift;
	
	#my $lowercase=lc($ortho);
	#my $PH=Text2Phoneme($lowercase);
	my $PH=$phonemisation;
	$PH=BugPreset($PH);
	my @ph=split(//,$PH);
	$PH=BugSet($PH);
	my $ph1=BugSet(pop(@ph));
	my $ph2=BugSet(pop(@ph));
	my $ph3=BugSet(pop(@ph));
	my $ph4=BugSet(pop(@ph));
	my $ph5=BugSet(pop(@ph));
	my $ph6=BugSet(pop(@ph));
	my $ph7=BugSet(pop(@ph));
	my $ph8=BugSet(pop(@ph));
	my $nbsyll=NbSyllabe($PH);
	my $requete="INSERT INTO dico ( ortho, lemme, cgram,nombre, genre, nbsyll, freq, PH,PH1,PH2,PH3,PH4,PH5,PH6,PH7,PH8 ) VALUES ( \"$ortho\", \"$lemme\", \"$cgram\",\"$nb\", \"$genre\", \"$nbsyll\",\"$freq\", \"$PH\" ,\"$ph1\",\"$ph2\",\"$ph3\",\"$ph4\",\"$ph5\",\"$ph6\",\"$ph7\",\"$ph8\" )";

	$db->do($requete) || print "Error : $requete\n";
}


sub LemmePresentDefinition
{
	my $mot=lc(shift);
	my $trouve=0;
	my $prep = $db->prepare("SELECT definition FROM definitions WHERE ortho='$mot'") or die $db->errstr;
	$prep->execute() or die "Echec requête\n";
	while ( my ($definition) = $prep->fetchrow_array )
	  {
		$trouve=1;
		}
	return  ( $trouve);
}

sub MotPresentDico
{
	my $mot=shift;
	
	my $trouve=0;
	my $prep = $db->prepare("SELECT ortho FROM dico WHERE ortho='$mot'") or die $db->errstr;
	$prep->execute() or die "Echec requête\n";
	while ( my ($definition) = $prep->fetchrow_array )
		{ 
		$trouve=1;
		}
	return  ( $trouve);
}
