#include "GMarchive.h"

#include <archive.h>
#include <archive_entry.h>
#include <QFileInfo>
#include <QDebug>


GMArchive::GMArchive(QString archivefilename, QObject *parent):QObject(parent)
{
    m_archive=archivefilename;
    m_dir=QFileInfo(m_archive).absolutePath();
    m_filename=QFileInfo(m_archive).baseName();
    m_errorNum=NoError;
    m_filesNumber=0;
    m_a=archive_read_new();
    archive_read_support_filter_all(m_a);
    archive_read_support_format_all(m_a);
    archive_read_support_format_empty(m_a);
    qWarning()<<"emit"<<m_archive<<QFileInfo(m_archive).size();
    emit totalSize(m_archive,QFileInfo(m_archive).size());


}

int GMArchive::getFilesNumber()
{
    if (m_archive.isEmpty()) return(0);
    int r;
    int count=0;
    r = archive_read_open_filename(m_a, m_archive.toStdString().c_str(), 10240);
    if (r != ARCHIVE_OK)
      {
          m_errorNum=ErrorReadFile;
          m_error=QString("archive_read_open_filename failed");
          return (0);
      }
    while (archive_read_next_header(m_a, &m_entry) == ARCHIVE_OK) {
         count++;
      archive_read_data_skip(m_a);
    }
    r = archive_read_free(m_a);
    if (r != ARCHIVE_OK)
      {
      m_errorNum=ErrorReadFree;
      m_error=QString("archive_read_free failed");
      return(0);
      }
    return count;
}

bool GMArchive::decompress()
{
      archive *a;
      archive *ext;
      archive_entry *entry;
      int flags;
      int r;

      /* Select which attributes we want to restore. */
      flags = ARCHIVE_EXTRACT_TIME;
      flags |= ARCHIVE_EXTRACT_PERM;
      flags |= ARCHIVE_EXTRACT_ACL;
      flags |= ARCHIVE_EXTRACT_FFLAGS;

      a = archive_read_new();
      archive_read_support_format_all(a);
      archive_read_support_compression_all(a);
      ext = archive_write_disk_new();
      archive_write_disk_set_options(ext, flags);
      archive_write_disk_set_standard_lookup(ext);
      if ((r = archive_read_open_filename(a, m_archive.toStdString().c_str(), 10240)))
      {
          m_error= QString("archive_read_open_filename failed");
          m_errorNum=ErrorReadFile;
          return false;
      }
      emit readsize(m_archive,10240);
      for (;;) {
        r = archive_read_next_header(a, &entry);
        if (r == ARCHIVE_EOF)
          break;
        if (r < ARCHIVE_OK)
        {
          m_error= QString("%1").arg(archive_error_string(a));
          m_errorNum=ErrorReadFormat;
          return false;
        }
        if (r < ARCHIVE_WARN)
          return false;
        r = archive_write_header(ext, entry);
        if (r < ARCHIVE_OK)
        {
          m_error= QString("%1").arg(archive_error_string(a));
          m_errorNum=ErrorReadFormat;
          return false;
         }
        else if (archive_entry_size(entry) > 0) {
          emit readsize(m_archive,archive_entry_size(entry));
          r = copy_data(a, ext);

          if (r < ARCHIVE_OK)
          {
            m_error= QString("%1").arg(archive_error_string(a));
            m_errorNum=ErrorReadFormat;
            return false;
          }
          if (r < ARCHIVE_WARN)
            return false;
        }
        r = archive_write_finish_entry(ext);
        if (r < ARCHIVE_OK)
        {
          m_error= QString("%1").arg(archive_error_string(a));
          m_errorNum=ErrorReadFormat;
          return false;
        }
        if (r < ARCHIVE_WARN)
          return false;
      }
      archive_read_close(a);
      archive_read_free(a);
      archive_write_close(ext);
      archive_write_free(ext);
      return true;
}

int GMArchive::copy_data( archive *a,  archive *ext)
{
      int r;
      const void *buff;
      size_t size;
      la_int64_t offset;

      for (;;) {
        r = archive_read_data_block(a, &buff, &size, &offset);
        emit readsize(m_archive,size);
        if (r == ARCHIVE_EOF)
          return (true);
        if (r < ARCHIVE_OK)
          return (false);
        r = archive_write_data_block(ext, buff, size, offset);
        if (r < ARCHIVE_OK) {
            {
              m_error= QString("%1").arg(archive_error_string(a));
              m_errorNum=ErrorReadFormat;
              return false;
            }
        }
      }
}



QString GMArchive::getError()
{
    return m_error;
}

GMArchive::ErrorNo GMArchive::getErrorNum()
{
    return m_errorNum;
}

QStringList GMArchive::getFileNames()
{
      QStringList ret;

      if (m_archive.isEmpty()) return(ret);
      int r;
      r = archive_read_open_filename(m_a, m_archive.toStdString().c_str(), 10240);
      if (r != ARCHIVE_OK)
        {
            m_errorNum=ErrorReadFile;
            m_error=QString("archive_read_open_filename failed");
            return (ret);
        }
      while (archive_read_next_header(m_a, &m_entry) == ARCHIVE_OK) {
        ret<<QString::fromUtf8(archive_entry_pathname_utf8(m_entry));
        archive_read_data_skip(m_a);
      }
      r = archive_read_free(m_a);
      if (r != ARCHIVE_OK)
        {
        m_errorNum=ErrorReadFree;
        m_error=QString("archive_read_free failed");
        return(ret);
        }
    return ret;
 }
