#include "oulipo.h"

#include <QtSql>
#include <QSqlQuery>
#include <QTime>
#include <QRegExp>
#include <QHash>
#include <QMessageBox>
#include <QSettings>

#include "wherecondition.h"
#include "phonemise.h"
#include "debug.h"




oulipo::oulipo()
{
	verbosity=0;
    QSettings s;
    s.setValue("DebugLevel",0);
	Pronoms<<"je"<<"tu"<<"il"<<"elle"<<"on"<<"nous"<<"vous"<<"ils"<<"elles";

}
int oulipo::Random ( int val)
{
	QTime timer = QTime::currentTime () ;
	int time=timer.msec();
	srand(time);
	return ( qrand()%val);
}

QString oulipo::TireHasard( QStringList  Liste)
{
	if ( Liste.isEmpty() ) return ( "") ;
	else
	return( Liste.at(Random(Liste.count())) );
}


QString oulipo::PersonnePronom( QString pronom)
{
	QHash<QString, QString> pp ;
	pp["je"]="1";
	pp["tu"]="2";
	pp["il"]="3";
	pp["elle"]="3";
	pp["on"]="3";
	pp["nous"]="4";
	pp["vous"]="5";
	pp["ils"]="6";
	pp["elles"]="6";
	return (pp[pronom]);

}
QString oulipo::ArticleMasculinSingulier(QString mot)
{
	QRegExp rx("^[aâäeéèëêiïîoôöuùüyAÂÄEËÊÉÈIÏÎÖÔUÜÛÙYhH]");
	//QRegExp rx("^[aeiouyAEIOUY]");
	QString le;
	if ( rx.indexIn(mot) !=-1 ) le="l'"; else le="le ";
	QStringList article;
	article << "un "<<le<<"mon "<<"ton "<<"son "<<"votre "<<"notre "<<"leur ";
	return article.at(Random(8));
}

QString oulipo::ArticleFemininSingulier(QString mot)
{
	QRegExp rx("^[aâäeéèëêiïîoôöuùüyAÂÄEËÊÉÈIÏÎÖÔUÜÛÙYHh]");
	//QRegExp rx("^[aeiouyAEIOUY]");
	//QRegExp rx("^a");

	QString la;
	if ( rx.indexIn(mot) !=-1) la="l'"; else la="la ";
	QStringList article;
	article << "une "<<la<<"ma" <<"ta "<<"sa "<<"votre "<<"notre "<<"leur ";
	return article.at(Random(8));
}

QString oulipo::ArticlePluriel(QString mot)
{
	Q_UNUSED(mot);
	QStringList article;
	article << "aux "<<"des "<<"les "<<"mes "<<"tes "<<"ses "<<"vos "<<"nos "<<"leurs ";
	return article.at(Random(8));
}

QString oulipo::ChercheArticle (QString mot )
{
	QString article;

	QString sql="SELECT genre,nombre FROM dico WHERE ( ortho = '"+mot+"' and cgram = 'NOM' ) LIMIT 1";
	debug::message(2,"ChercheArticle : "+sql);
	QSqlQuery query1(sql);

	if ( query1.next() )
	{
		QString genre=query1.value(0).toString();
		QString pluriel=query1.value(1).toString();
		if ( pluriel=="s" ) {
		if ( genre == "f" ) { article=ArticleFemininSingulier(mot) ;}
		else  { article=ArticleMasculinSingulier(mot);}
		}
		else  article=ArticlePluriel(mot);
	}
	else
	{
		article="";
	}
	return (article);
}

QString oulipo::ChercheArticleAdjectif( QString mot )
{

QString article;

	QString sql="SELECT genre,nombre FROM dico WHERE ( ortho = '"+mot+"' and ( cgram = 'ADJ' or cgram = 'VER'  )) LIMIT 1";
	debug::message(2,"ChercheArticleAdjectif : "+sql);
	QSqlQuery query1(sql);

	if ( query1.next() )
	{
		QString genre=query1.value(0).toString();
		QString pluriel=query1.value(1).toString();
		if ( pluriel=="s" ) {
		if ( genre == "f" ) { article=ArticleFemininSingulier(mot) ;}
		else  { article=ArticleMasculinSingulier(mot);}
		}
		else  article=ArticlePluriel(mot);
	}
	else
	{
		article="";
	}
	return (article);
}

QString oulipo::ConjugueVerbe(  QString verbe, QString personne )
{
	QString sql="SELECT lemme FROM dico WHERE ( ortho = '"+verbe+"' and cgram = 'VER' ) LIMIT 1";
	QSqlQuery query(sql);
	query.next();
	QString lemme= query.value(0).toString();
	QSqlQuery query1("SELECT ortho FROM dico where ( lemme = '"+lemme+"' and cgram = 'VER' and genre LIKE '%%"+personne+"A%%' )");
	query1.next();
	return ( query1.value(0).toString());
}

QString oulipo::DetermineType ( QString mot )
{
	QString type;
	QString sql="SELECT cgram FROM dico WHERE ( ortho = '"+mot+"' ) LIMIT 1";
	QSqlQuery query1(sql);

	if ( query1.next() )
	{	type=query1.value(0).toString(); }

	return (type);
}

QString oulipo::DetermineGenre ( QString mot)
{
	QString genre;
	QString sql="SELECT genre FROM dico WHERE ( ortho = '"+mot+"' ) LIMIT 1";
	QSqlQuery query1(sql);

	if ( query1.next() )
	{	genre=query1.value(0).toString(); }
	if ( genre != "m" && genre != "f"  ) genre ="m" ;
	return (genre);
}

QString oulipo::DetermineNombre ( QString mot )
{
	QString type;
	QString sql="SELECT nombre FROM dico WHERE ( ortho = '"+mot+"' and cgram = 'NOM'  LIMT 1)";
	QSqlQuery query1(sql);

	if ( query1.next() )
	{	type=query1.value(0).toString(); }

	if ( type != "p" && type != "s" ) type ="s";
	return (type);
}


QStringList oulipo::ChercheParties ( QString phoneme,int longueur)
{
	QStringList PhonemeList;
	for (int j=0;j<phoneme.length()-longueur+1;j++)
	{
		// j est la longueur du mot
		QString mot=phoneme;
		mot.remove(j+longueur, phoneme.length()-j);
		mot.remove(0, j);
		PhonemeList.append(mot);
 }
	return (PhonemeList);

}

QString oulipo::Where( QString mot,int longueur)
{
	int L=longueur;
	phonemise ph;
	QString phoneme=ph.Text2Phoneme(mot);
	if ( L > phoneme.length() ) L =phoneme.length() ;
	QStringList List=ChercheParties(phoneme,L);
	QStringList List2;
	foreach ( QString l,List) { List2.append("PH LIKE '%%"+l+"%%'") ;}
	QString ret= List2.join(" OR ");
	return(" (" + ret + ")");
}

QString oulipo::requete( QString mot,int longueur, QString type)
{
	QString ret="SELECT ortho FROM dico WHERE ( " + Where(mot,longueur) + " AND cgram = '"+ type + "' )";
	return (ret);
}


void oulipo::ChercheNom ( QString mot, int NbSyl )
{
	   QSqlQuery query(requete(mot,NbSyl,"NOM"));
	   while  ( query.next() )  { Noms.append(query.value(0).toString() );}

}

void oulipo::CherchePrenom ( QString mot, int NbSyl )
{
	   QString req=requete(mot,NbSyl,"PRENOM");
	   debug::message(2,"CherchePrenom : "+req);
	   QSqlQuery query(req);
	   while  ( query.next() )  { Prenoms.append(query.value(0).toString() );}
}

void oulipo::ChercheVerbe ( QString mot, QString Personne, int NbSyl )
{

	 QString req="SELECT ortho FROM dico WHERE ( " + Where(mot,NbSyl) + " AND cgram = 'VER' AND genre   LIKE \"%%"+Personne+"%%\" )";
	 debug::message(2,"ChercheVerbe : "+req);
	 QSqlQuery query(req);
		 while  ( query.next() )  { Verbes.append(query.value(0).toString() );}
}

void oulipo::ChercheAdjectif(QString mot, int NbSyl)
{
	QString genre=DetermineGenre(mot);
	QString nombre=DetermineNombre(mot);
	if (nombre.isEmpty()) nombre = "s";
	QString req="SELECT ortho FROM dico WHERE ( " + Where(mot,NbSyl) + " AND ( ( cgram = 'ADJ' OR  cgram = 'VER' ) AND genre = \""+genre+"\" AND nombre =  \""+nombre+"\") ) ";
	debug::message(2,"ChercheAdjectif : "+req);
	QSqlQuery query(req);
	   while ( query.next() ) { Adjectifs.append(query.value(0).toString() );}

}

void oulipo::ChercheAdverbe(QString mot, int NbSyl)
{
	   QSqlQuery query(requete(mot,NbSyl,"ADV"));
	   while ( query.next() ){ Adverbes.append(query.value(0).toString() );}
}


QString oulipo::ComposePhrase( QString mot, int NbSyl , QString typephrase)
{
	QString phrase ;
	bool nomplace=false;
	bool listenom=false;
	QString Adverbe,Verbe,Prenom,Nom1,Nom2,Genre1,Genre2,Nombre1,Nombre2,Article1,Article2,Adjectif1,Adjectif2;
	QString personne;
	QStringList mots=typephrase.split(" ");
	phrase=mots.join( " _" );
	phrase="_"+phrase;
	QString type=DetermineType(mot);
	if ( phrase.contains("_pronom") ) {
				QString pron;
				if ( PersonnePronom(mot)  != "" ) pron=mot;
				else pron=TireHasard(Pronoms);
				phrase.replace("_pronom",pron) ; personne=PersonnePronom(pron);
			}
	if ( phrase.contains(QString::fromUtf8("_prénom _verbe"))) personne="3";
	if ( phrase.contains("_adverbe")){  if ( type == "ADV" ) Adverbe= mot ; else  { ChercheAdverbe(mot,NbSyl) ; Adverbe= TireHasard(Adverbes); } }
	phrase.replace("_adverbe",Adverbe);
	if ( phrase.contains("_article _adjectif _nom _verbe"))
	{
	   if ( type == "NOM") { Nom1=mot;nomplace=true;} else {listenom=true;ChercheNom(mot,NbSyl); Nom1=TireHasard(Noms);}
	   if ( DetermineNombre(Nom1) == "p" ) personne="6" ; else personne="3";
	   if ( type == "ADJ" ) { Adjectif1=mot;} else {ChercheAdjectif(mot,NbSyl);Adjectif1=TireHasard(Adjectifs);}
	   Article1=ChercheArticleAdjectif(Adjectif1);

	   phrase.replace("_article _adjectif _nom _verbe",Article1 + " "+Adjectif1+ " "+Nom1+" _verbe");
	}
	if ( phrase.contains("_article _nom _adjectif _verbe"))
	{
	   if ( type == "NOM") { Nom1=mot;nomplace=true;} else {listenom=true;ChercheNom(mot,NbSyl); Nom1=TireHasard(Noms);}
	   if ( DetermineNombre(Nom1) == "p" ) personne="6" ; else personne="3";
	   if ( type == "ADJ" ) { Adjectif1=mot;} else { ChercheAdjectif(mot,NbSyl);Adjectif1=TireHasard(Adjectifs);}
	   Article1=ChercheArticle(Nom1);

	   phrase.replace("_article _nom _adjectif _verbe",Article1 + " "+Nom1+" "+Adjectif1+" _verbe");
	}
	if ( phrase.contains("_article _nom _verbe"))
	{
	   if ( type == "NOM") { Nom1=mot; nomplace=true;} else {listenom=true;ChercheNom(mot,NbSyl); Nom1=TireHasard(Noms);}
	   if ( DetermineNombre(Nom1) == "p" ) personne="6" ; else personne="3";
	   Article1=ChercheArticle(Nom1);

	   phrase.replace("_article _nom _verbe",Article1 +  " "+Nom1+" _verbe");
	}
	if ( type == "VER" ) Verbe=ConjugueVerbe(mot,personne); else { ChercheVerbe(mot,personne,NbSyl);Verbe=TireHasard(Verbes);}
	phrase.replace("_verbe",Verbe);


	if ( phrase.contains(QString::fromUtf8("_prénom"))) { if ( type == "PRENOM" ) Prenom=mot ; else { CherchePrenom(mot,NbSyl); Prenom=TireHasard(Prenoms); } }
	phrase.replace(QString::fromUtf8("_prénom"),Prenom);
	if ( phrase.contains("_article _nom"))
	{
	   if ( type == "NOM" && ! nomplace ) { Nom2=mot;} else { if ( ! listenom ) ChercheNom(mot,NbSyl); Nom2=TireHasard(Noms);}
	   Article2=ChercheArticle(Nom2);
	   phrase.replace("_article _nom",Article2+" "+Nom2);
   }

	return ( phrase) ;
}


int oulipo::NombrePossibilite()
{
	int nom= Noms.count() ;
	if (nom==0) nom=1;
	int pren=Prenoms.count();
	if (pren==0) pren=1;
	int ver= Verbes.count();
	if ( ver==0) ver=1;
	int adj=Adjectifs.count();
	if ( adj==0) adj=1;
	int adv=Adverbes.count();
	if ( adv==0) adv=1;
	int pron=Pronoms.count();
	if (pron==0) pron=1;
	return ( nom*pren*ver*adj*adv*pron);
}
