/***************************************************************************
 *   Copyright (C) 2007-2014 by Gilles Maire   *
 *   Gilles@lesjetesdelencre.com                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#include <QtSql>
#include <QHeaderView>
#include <QSqlQuery>
#include <QStringList>
#include <QListView>
#include <QSettings>
#include <QClipboard>
#include <QApplication>
#include <QItemSelectionModel>
#include <QModelIndex>
#include <QCursor>
#include <QRegExp>
#include <QPrinter>
#include <QPrintDialog>
#include <QPrintPreviewDialog>
#include <QPrinterInfo>
#include <QMessageBox>
#include <QFileDialog>
#include <QDebug>
#include <QTimer>
#include <QLocale>
#include <QCompleter>
#include <lanquage.h>


#include "qdictionnaire.h"
#include "phonemise.h"
#include "ui_citation.h"
#include "nouveautes.h"
#include "ui_sourcedefinition.h"
#include "oulipo.h"
#include "wherecondition.h"
#include "conjugaison.h"
#include "formatage.h"
#include "debug.h"
#include "phonemisation.h"
#include "correctionorthographique.h"
#include "feuillestyle.h"
#include "synonymes.h"
#include "version.h"
#include "dialogsysteminfo.h"
#include "eula.h"


//Constructeur
qdictionnaire::qdictionnaire ( QWidget* parent )
                : QMainWindow ( parent ),ui(new Ui::Dicorime)
{
        ui->setupUi(this);

        debug=false;

        EditChange=false;
        debug::message ( 1,"constructeur : lancement");
        QSettings settings;

        if ( settings.value("WindowGeometry/SaveMainWindowGeometry").toBool())
        {
            if (! settings.value("WindowGeometry/width").isNull() && !settings.value("WindowGeometry/height").isNull())
            {
                QSize size;
                size.setHeight(settings.value("WindowGeometry/height").toInt());
                size.setWidth(settings.value("WindowGeometry/width").toInt());
                this->resize(size);
                ui->splitterPrincipal->restoreState(settings.value("WindowGeometry/splitterstage").toByteArray());
                ui->splitterDroite->restoreState(settings.value("WindowGeometry/splitterdroite").toByteArray());
                ui->splitterVertical->restoreState(settings.value("WindowGeometry/splittervertical").toByteArray());
            }

        }
        ui->splitterPrincipal->setCollapsible(2, false);
        ui->actionListe_des_rimes->setChecked(ui->splitterPrincipal->sizes().at(1)>5);
        ui->actionType_de_rimes->setChecked(ui->splitterPrincipal->sizes().at(0)>5);

        MasqueAfficheFleches(0,0);
        m_LigneRimeCourante=-1;     ///
        /// \brief makeToolBar construit la barre de rechercher, avec les différents champs activés etc...
        ///

        m_configuration= new configuration(this);
        m_apropos = new QDialog(this);
        m_configuration->AppPath=AppPath;

        m_CompleterLigneCherche = new QCompleter(this);
        m_ModelCompleter = new QStringListModel(m_ListeDerniersMots);
        m_CompleterLigneCherche->setModel(m_ModelCompleter);
        ui->lineEditMotCherche->setCompleter(m_CompleterLigneCherche);

        ui->tableViewListeReponse->setSelectionBehavior(QAbstractItemView::SelectRows);
        ui->tableViewListeReponse->setSelectionMode(QAbstractItemView::SingleSelection);
        mSystemTray = new QSystemTrayIcon(this);
        mSystemTray->setIcon(QIcon(QPixmap(":/images/qdictionnaire32.png")));
        mMenu = new QMenu(this);
        mMenu->addAction("Affiche/Cache qDictionnaire", this, SLOT(AfficheMasque()));
        mMenu->addAction("Quitter", qApp, SLOT(quit()), QKeySequence::Quit);
        mSystemTray->setContextMenu(mMenu);
        mSystemTray->show();
        connect(mSystemTray,SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this,SLOT(Activation(QSystemTrayIcon::ActivationReason)));

        ui->spinBoxRichessRime->setValue(settings.value("RichesseRimeDefaut").toInt());
        ui->checkBoxLieux->setChecked(settings.value("AfficherLieux").toInt()==1);
        ui->checkBoxPrenoms->setChecked(settings.value("AfficherPrenoms").toInt()==1);
        ui->checkBoxAdjectifs->setChecked(settings.value("AfficherAdjectifs").toInt()==1);
        ui->checkBoxVerbes->setChecked(settings.value("AfficherVerbes").toInt()==1);
        ui->checkBoxSigles->setChecked(settings.value("AfficherSigles").toInt()==1);
        ui->checkBoxNoms->setChecked(settings.value("AfficherNoms").toInt()==1);

        mCorrecteur= new CorrectionOrthographique;

        m_rimeModel = new CustomSqlModel(this);

        //	model->Phoneme=phoneme;
        m_rimeProxyModel = new QSortFilterProxyModel(this);
        m_rimeProxyModel->setSourceModel(m_rimeModel);
        ui->tableViewListeReponse->setModel(m_rimeProxyModel);

        //connexion des boutons et des combo box
        connect ( ui->toolButtonEffacer, SIGNAL ( clicked()), this, SLOT ( EffacerBouton_clicked()));
        connect ( ui->spinBoxRichessRime, SIGNAL ( valueChanged(int)), this, SLOT ( AfficherRimeParNbPhoneme()));
        connect ( ui->pushButtonRechercheLibreCitation, SIGNAL(clicked()),this,SLOT(ChercherLibreCitationAuteur()));
        connect ( ui->lineEditRechercheLibreCitation,SIGNAL(returnPressed()),this,SLOT(ChercherLibreCitationAuteur()));
        connect ( ui->comboBoxTypeRime, SIGNAL(activated(int)),this,SLOT (AfficherRimeParType()));
        connect ( ui->lineEditMotCherche, SIGNAL (returnPressed ()), this, SLOT ( AfficheTabulation()));
        connect ( ui->textEditCodeEditor, SIGNAL ( textChanged () ), this , SLOT ( EditeurChange()));
        connect ( ui->pushButtonPhonemisation,SIGNAL( clicked()),this,SLOT(AffichagePhonemisation()));
        connect ( ui->pushButtonCorrection,SIGNAL(clicked()),this,SLOT(Correction()));

        //connect (ui->tabWidgetMode,SIGNAL(currentChanged(int)),SLOT(equalizeCitationsColumns()));

        connect (ui->tabWidgetMode,SIGNAL(currentChanged(int)),SLOT(AfficheTabulation(int)));
        connect (ui->tableViewListeReponse,SIGNAL(clicked(QModelIndex)),this,SLOT(AfficheTabulation(QModelIndex)));
        connect (ui->tableViewListeReponse,SIGNAL(Next()),this,SLOT(Next()));
        connect (ui->tableViewListeReponse,SIGNAL(Previous()),this,SLOT(Previous()));

        //Connexion des menus
        connect ( ui->actionConfiguration_2 , SIGNAL (triggered () ), this, SLOT (configurationdef()));
        connect ( ui->actionApropos , SIGNAL (triggered () ), this, SLOT (apropos()));
        connect ( ui->actionAide , SIGNAL (triggered () ), this, SLOT (aide()));
        connect ( ui->actionImpression, SIGNAL (triggered()),this,SLOT(impression()));
        connect ( ui->actionStatistiques, SIGNAL ( triggered() ), this , SLOT ( Statistiques() ));
        connect ( ui->actionOuvrir, SIGNAL ( triggered() ), this , SLOT ( OuvrirFichier() ));
        connect ( ui->actionEnregistrer, SIGNAL ( triggered() ), this , SLOT ( SauverFichier() ));
        connect ( ui->actionEnregistrerSous, SIGNAL ( triggered() ), this , SLOT ( SauverFichierSous() ));
        connect ( ui->actionNouveau, SIGNAL ( triggered()),this,SLOT(NouveauFichier()));
        connect ( ui->actionNouveaute,SIGNAL(triggered()),this,SLOT(ActionNouveautes()));
        connect (ui->actionInformations_syst_mes,SIGNAL(triggered()),SLOT(InformationsSystemes()));
        connect (ui->actionLicence,SIGNAL(triggered(bool)),this,SLOT(Licence()));

        connect (ui->pushButtonInserer, SIGNAL ( clicked()), this, SLOT( FabriquePhrase() )) ;
        connect (ui->comboBoxTypePhrase,SIGNAL(currentIndexChanged(int)),this,SLOT( FabriquePhrase() )) ;

        connect (ui->checkBoxLieux,SIGNAL(clicked()),this,SLOT(AfficherRimeParType()));
        connect (ui->checkBoxPrenoms,SIGNAL(clicked()),this,SLOT(AfficherRimeParType()));
        connect (ui->checkBoxAdjectifs,SIGNAL(clicked()),this,SLOT(AfficherRimeParType()));
        connect (ui->checkBoxVerbes,SIGNAL(clicked()),this,SLOT(AfficherRimeParType()));
        connect (ui->checkBoxNoms,SIGNAL(clicked()),this,SLOT(AfficherRimeParType()));
        connect (ui->checkBoxSigles,SIGNAL(clicked()),this,SLOT(AfficherRimeParType()));


        connect (  ui->textEditCodeEditor,SIGNAL(AskCitation(QString)),this,SLOT(AskCitations(QString)));
        connect (  ui->textEditCodeEditor,SIGNAL(AskDefinition(QString)),this,SLOT(AskDefinition(QString)));
        connect (  ui->textEditCodeEditor,SIGNAL(AskConjugaison(QString)),this,SLOT(AskConjugaison(QString)));
        connect (  ui->textEditCodeEditor,SIGNAL(AskRime(QString)),this,SLOT(AskRime(QString)));

        connect (  ui->textBrowserDefinition,SIGNAL(AskCitation(QString)),this,SLOT(AskCitations(QString)));
        connect (  ui->textBrowserDefinition,SIGNAL(AskDefinition(QString)),this,SLOT(AskDefinition(QString)));
        connect (  ui->textBrowserDefinition,SIGNAL(AskRime(QString)),this,SLOT(AskRime(QString)));

        connect (  ui->textBrowserEtymologie,SIGNAL(AskCitation(QString)),this,SLOT(AskCitations(QString)));
        connect (  ui->textBrowserEtymologie,SIGNAL(AskDefinition(QString)),this,SLOT(AskDefinition(QString)));
        connect (  ui->textBrowserEtymologie,SIGNAL(AskRime(QString)),this,SLOT(AskRime(QString)));

        connect (  ui->textBrowserExpressions,SIGNAL(AskCitation(QString)),this,SLOT(AskCitations(QString)));
        connect (  ui->textBrowserExpressions,SIGNAL(AskDefinition(QString)),this,SLOT(AskDefinition(QString)));
        connect (  ui->textBrowserExpressions,SIGNAL(AskRime(QString)),this,SLOT(AskRime(QString)));


        connect ( ui->pushButtonSourceWiktionary,SIGNAL(clicked()),this,SLOT(MontreSourceWiktionary()));

        connect (ui->toolButtonHome,SIGNAL(clicked(bool)),this,SLOT(Home()));
        connect (ui->toolButtonEnd,SIGNAL(clicked(bool)),this,SLOT(End()));
        connect (ui->toolButtonNext,SIGNAL(clicked(bool)),this,SLOT(Next()));
        connect (ui->toolButtonPrevious,SIGNAL(clicked(bool)),this,SLOT(Previous()));
        connect (ui->toolButtonPageNext,SIGNAL(clicked(bool)),this,SLOT(PageNext()));
        connect (ui->toolButtonPagePrevious,SIGNAL(clicked(bool)),this,SLOT(PagePrevious()));
        connect(ui->splitterPrincipal,SIGNAL(splitterMoved(int,int)),this,SLOT(MasqueAfficheFleches(int,int)));
        connect (qApp,SIGNAL(aboutToQuit()),this,SLOT(AuRevoir()));
        connect (ui->actionType_de_rimes,SIGNAL(toggled(bool)),this,SLOT(SetSplitterTypeRime(bool)));
        connect (ui->actionListe_des_rimes,SIGNAL(toggled(bool)),this,SLOT(SetSplitterListeRimes(bool)));

        connect ( m_configuration,SIGNAL(ReloadConfig()),this,SLOT(ReloadConfig()));
        connect ( m_configuration,SIGNAL(SaveGeometry()),this,SLOT(SauveGeometry()));
        connect (m_configuration,SIGNAL(ChangeLanguages(QTranslator*,QTranslator*)),this, SLOT(ChangeLangs(QTranslator*,QTranslator *)));

        ui->tabWidgetMode->setCurrentIndex(settings.value("Mode").toInt());

        if (settings.value("OpenLastFileOpen").toBool())  OuvrirDernierFichier();

        NbCitations=0;
        NbRimes =0 ;
        NbLignesEditeur=0;
        OngletVerbe=false;
        ConnecteursCitation=false;
        EditChange=false;
        AfficheMenuListeFichier();
        QTimer *timerSecond = new QTimer(this); // qu'une seule fois par appli ainsi que les suivantes
        connect(timerSecond, SIGNAL(timeout()), this, SLOT(EachSecondCheck()));
        timerSecond->start(1000);
}


void qdictionnaire::AfficheMasque()
{
    if (this->isVisible()) this->hide();
    else show();
}

void qdictionnaire::MontreSourceWiktionary()
{

    QDialog *window =new QDialog(this);
    Ui::SourceDefinition sd;
    sd.setupUi(window);
    window->setModal(true);
    window->show();
    QString requete="SELECT definition FROM definitions WHERE ortho = '"+ ui->labelMotTrouveInfo->text()+"'"  ;
    QSqlQuery query(requete);
    query.next();
    sd.SourceWiktionary->setText(query.value(0).toString());
}

bool qdictionnaire::Approchant ( QString phoneme1, QString phoneme2)
{
phoneme1.replace("O","o");
phoneme2.replace("O","o");
phoneme1.replace("Â°","");
phoneme2.replace("Â°","");
phoneme2.replace("E","e");
phoneme1.replace("E","e");
// diksjOnER:diksjonER"
return( phoneme1==phoneme2);
}




QString qdictionnaire::DefinitionVerbeConjugue ( QString codecomplet, QString verbe)
{
        QString retour;
        debug::message(4,"code complet:"+codecomplet);
        for ( int i=0 ; i<codecomplet.length(); i+=2)
        {
                formatage F;
                retour+=F.EtendChiffreVerbe(codecomplet.at(i));
                retour+=F.EtendLettreVerbe(codecomplet.at(i+1),verbe);
        }
        return(retour);
}

void qdictionnaire::AddMenuEdit( QString mot )
{
    QSettings set;
    if ( ! mot.isEmpty() && ! m_ListeDerniersMots.contains(mot))  m_ListeDerniersMots.append(mot);
    if (m_ListeDerniersMots.length() > set.value("NombreDerniersMotsEntres").toInt())
        m_ListeDerniersMots.pop_front();
    m_ModelCompleter->setStringList(m_ListeDerniersMots);
    m_CompleterLigneCherche->setModel(m_ModelCompleter);
}


void qdictionnaire::ConstruitListeAnagrammes ( QString text )
{
  QString wherecondition = WhereAnagramme (text) ;
  QString requete="SELECT ortho,ID FROM dico WHERE ( "+wherecondition+" )";
  debug::message(2,"Req110 Anagramme  : " + requete);
  ui->comboBoxAnagrammes->clear();
  AnagrammesList.clear();
  ui->comboBoxAnagrammes->disconnect();
  QSqlQuery query(requete);
  QStringList liste;
  QMap<QString,QString> listeindex ;

  while (query.next()) {
           if ( ExtraCaractere(query.value(0).toString(),text)) {
                       liste.append(query.value(0).toString());
                       listeindex[query.value(0).toString()]=query.value(1).toString();

           }
  }
  liste.removeDuplicates();
  foreach ( QString l, liste) { AnagrammesList.append(listeindex[l]);}
  ui->comboBoxAnagrammes->addItems(liste);
  ui->comboBoxAnagrammes->setCurrentIndex(ui->comboBoxAnagrammes->findText(ui->labelMotTrouveInfo->text()));
  connect ( ui->comboBoxAnagrammes, SIGNAL(currentIndexChanged(int)), this, SLOT (RelancerRechercheAnagramme()));
}



void qdictionnaire::ConstruitListeHomophones ( QString text, QString type )
{
  phonemise mot;
  QSettings set;
  int fontsize=set.value("TaillePoliceDefaut").toInt();
  mot.UnsetDebug();
  QString phoneme= mot.Text2Phoneme(text);
  QString requete="SELECT ortho,cgram,ID FROM dico WHERE ( PH=\""+ phoneme +"\")";
  ui->comboBoxHomophones->clear();
  ui->comboBoxHomophones->disconnect();

  QSqlQuery query(requete);
  int index=0;
  int sizemax=0;
  while (query.next()) {
                 QString item=query.value(0).toString()+"["+query.value(1).toString()+"]";
                 ui->comboBoxHomophones->addItem(item);
                 HomophonesList.append(query.value(2).toString());
                 if ( query.value(0).toString() == text && query.value(1).toString() == type ) { ui->comboBoxHomophones->setCurrentIndex(index);}
                 index++;
                 sizemax=item.length()>sizemax?item.length():sizemax;
  }
 ui->comboBoxHomophones->view()->setMinimumWidth(sizemax*fontsize);
 connect ( ui->comboBoxHomophones, SIGNAL(currentIndexChanged(int)), this, SLOT (RelancerRechercheHomophones()));
}


void qdictionnaire::ConstruitListeHomographes ( QString text , QString type)
{
        ui->comboBoxHomographes->clear();
        ui->comboBoxHomographes->disconnect();
        QSqlQuery query("SELECT ortho,cgram,ID FROM dico WHERE ( ortho=\""+ text +"\")");
        int index=0;
        while (query.next()) {
                 ui->comboBoxHomographes->addItem(query.value(0).toString()+"["+query.value(1).toString()+"]");
                 HomographesList.append(query.value(2).toString());
                 if ( query.value(0).toString() == text && query.value(1).toString() == type ) { ui->comboBoxHomographes->setCurrentIndex(index);}
                 index++;
  }
 connect ( ui->comboBoxHomographes, SIGNAL(currentIndexChanged(int)), this, SLOT (RelancerRechercheHomographes()));
}



void qdictionnaire::AfficheTabulation()
{
        m_motcherche=ui->lineEditMotCherche->text();
        AddMenuEdit(ui->lineEditMotCherche->text());
        MiseAJourHaut();
        AfficheRime();
        if ( ui->tabWidgetMode->currentIndex() == Onglet::Definition)
            AfficheDefinition();
        else if ( ui->tabWidgetMode->currentIndex() == Onglet::Citation)
            AfficheCitation();
        else if (ui->tabWidgetMode->currentIndex() == Onglet::Conjugaisons)
            AfficheConjugaison();
        else if (ui->tabWidgetMode->currentIndex() == Onglet::Edition)
            AfficheEditeur();
        else if (ui->tabWidgetMode->currentIndex()==Onglet::InfoRime)
            AfficheInfoRime();
}

void qdictionnaire::AfficheTabulation(int index)
{
    m_motcherche=ui->lineEditMotCherche->text();
    AddMenuEdit(ui->lineEditMotCherche->text());
    qWarning()<<IndexDefinition;
    MiseAJourHaut();
    AfficheRime();
    if ( index == Onglet::Definition)
        AfficheDefinition();
    else if ( index == Onglet::Citation)
        AfficheCitation();
    else if (index == Onglet::Conjugaisons)
        AfficheConjugaison();
    else if (index == Onglet::Edition)
        AfficheEditeur();
    else if (index==Onglet::InfoRime)
        AfficheInfoRime();
}

void qdictionnaire::AfficheTabulation(QModelIndex index)
{
    if ( !index.isValid() ) return;
    m_motcherche=index.data().toString();
    ui->lineEditMotCherche->setText(m_motcherche);
    MiseAJourHaut();
    AfficheRime();
    if ( ui->tabWidgetMode->currentIndex() == Onglet::Definition)
        AfficheDefinition();
    else if ( ui->tabWidgetMode->currentIndex() == Onglet::Citation)
        AfficheCitation();
    else if ( ui->tabWidgetMode->currentIndex() == Onglet::Conjugaisons)
        AfficheConjugaison();
    else if ( ui->tabWidgetMode->currentIndex() == Onglet::Edition)
        AfficheEditeur();
    else if ( ui->tabWidgetMode->currentIndex() == Onglet::InfoRime)
        AfficheInfoRime();
}

void qdictionnaire::AfficheTabulation( QModelIndex selected, QModelIndex )
{
    m_motcherche=selected.data().toString();
    ui->lineEditMotCherche->setText(m_motcherche);
    MiseAJourHaut();
    AfficheRime();
    if ( ui->tabWidgetMode->currentIndex() == Onglet::Definition)
        AfficheDefinition();
    if ( ui->tabWidgetMode->currentIndex() == Onglet::Citation)
        AfficheCitation();
    if ( ui->tabWidgetMode->currentIndex() == Onglet::Conjugaisons)
        AfficheConjugaison();
    else if (ui->tabWidgetMode->currentIndex() == Onglet::Edition)
        AfficheEditeur();
    else if (ui->tabWidgetMode->currentIndex()== Onglet::InfoRime)
        AfficheInfoRime();
}



void qdictionnaire::AfficheInfoRime()
{
    QString requete=QString("SELECT ortho,lemme,cgram,genre,nombre,nbsyll,freq,PH FROM dico WHERE ( ID = \"%1\")").arg(IndexDefinition);
    QSqlQuery query(requete);
    query.next();
    ui->lineEditOrthographe->setText(query.value("ortho").toString());
    ui->lineEditLemme->setText(query.value("lemme").toString());
    ui->lineEditCgram->setText(query.value("cgram").toString());
    ui->lineEditGenre->setText(query.value("genre").toString());
    ui->lineEditNombre->setText(query.value("nombre").toString());
    ui->lineEditSyllabes->setText(query.value("nbsyll").toString());
    ui->lineEditFrequence->setText(query.value("freq").toString());
    ui->lineEditPhonetique->setText(query.value("PH").toString());
    ui->lineEditId->setText(IndexDefinition);
}

void qdictionnaire::equalizeCitationsColumns()
{
    ui->tableViewCitations->setColumnWidth ( 0, (int)(7*ui->tableViewCitations->width()/50) );
    ui->tableViewCitations->setColumnWidth ( 1, (int)(7*ui->tableViewCitations->width()/50) );
    ui->tableViewCitations->setColumnWidth ( 2, (int)(28*ui->tableViewCitations->width()/50) );
    ui->tableViewCitations->setColumnWidth ( 3, (int)(7*ui->tableViewCitations->width()/50) );
}

void qdictionnaire::equalizeReponsesColumns()
{
    ui->tableViewListeReponse->setColumnHidden(1,true);
    ui->tableViewListeReponse->setColumnHidden(2,true);
    ui->tableViewListeReponse->setColumnHidden(3,true);
    ui->tableViewListeReponse->setColumnHidden(4,true);
    ui->tableViewListeReponse->setColumnHidden(5,true);
    ui->tableViewListeReponse->setColumnHidden(6,true);
    ui->tableViewListeReponse->setColumnHidden(7,true);
    ui->tableViewListeReponse->setColumnHidden(8,true);
}

void qdictionnaire::resizeEvent(QResizeEvent *)
{
    equalizeReponsesColumns();
    equalizeCitationsColumns();
    ui->lineEditMotCherche->setFocus();


}

void qdictionnaire::AfficheRime()
{
//    if (  m_LigneRimeCourante != -1 )
//    {
//        ui->tableViewListeReponse->selectionModel()->select(m_ModelRimeCourante,QItemSelectionModel::Select);
//    }
    int nbtrouve;
        if ( !ui->labelMotTrouveInfo->text().isEmpty() || !ui->lineEditMotCherche->text().isEmpty() )
        {
        QString Where;
        wherecondition WC ;
        Where=WC.WhereCondition(ui->labelPhonemisationInfo->text(),ui->comboBoxTypeRime->currentText(),ui->spinBoxRichessRime->value());
        QString LieuxCondition ;
        if ( ! ui->checkBoxLieux->isChecked()) LieuxCondition=" AND cgram != 'VIL' AND cgram != 'DEP' AND cgram != 'PAY'" ;
        QString PrenomsCondition ;
        if ( ! ui->checkBoxPrenoms->isChecked()) PrenomsCondition=" AND cgram != 'PRN'" ;
        QString VerbesCondition ;
        if ( ! ui->checkBoxVerbes->isChecked()) VerbesCondition=" AND ( cgram != 'VER' OR cgram = lemme )" ;
        QString AdjectifsCondition ;
        if ( ! ui->checkBoxAdjectifs->isChecked()) AdjectifsCondition=" AND cgram != 'ADJ'" ;
        QString SiglesCondition ;
        if ( ! ui->checkBoxSigles->isChecked()) SiglesCondition=" AND cgram != 'SIG'" ;
        QString NomsCondition ;
        if ( ! ui->checkBoxNoms->isChecked()) NomsCondition=" AND cgram != 'NOM'" ;


        QString requete="SELECT ortho,nbsyll,genre,cgram,lemme,PH,freq,ID,Nombre FROM dico WHERE ( "+Where+PrenomsCondition+LieuxCondition+VerbesCondition+AdjectifsCondition+SiglesCondition+NomsCondition+" ) ORDER BY length(PH)";
        QString requetecount="SELECT count(ortho) FROM dico WHERE ("+Where+PrenomsCondition+LieuxCondition+VerbesCondition+AdjectifsCondition+SiglesCondition+NomsCondition+" )";
        QSqlQuery query(requetecount);
        query.next();
        ui->labelNombreMotTrouveInfo->setText(query.value(0).toString());
        nbtrouve=query.value(0).toInt();
        m_rimeModel->m_total=nbtrouve;
        ui->tableViewListeReponse->setShowGrid (false );
        ui->tableViewListeReponse->setSortingEnabled ( true );
        m_rimeProxyModel->setSourceModel(m_rimeModel);
        ui->tableViewListeReponse->setModel(m_rimeProxyModel);

        m_rimeModel->setQuery(requete);

        equalizeReponsesColumns();

        connect ( ui->tableViewListeReponse, SIGNAL(clicked(QModelIndex)), this, SLOT (DefinitionMot()));

        // 	 ListeReponse->resizeColumnToContents ( 6) ;
        QString count= ui->labelNombreMotTrouveInfo->text()+ QString::fromUtf8(" trouvée") ;
        if (nbtrouve > 1) count+="s";
        ui->labelNombreMotTrouveInfo->setText(count );
        // aller sur le mot dans les rimes
        int i =0;
        while ( ui->tableViewListeReponse->model()->index(i,0).data().toString()!=m_motcherche && i != nbtrouve -1)
        { i++;}
        ui->tableViewListeReponse->selectRow(i);

        ui->tableViewListeReponse->scrollTo(ui->tableViewListeReponse->model()->index(i,0));


 }
}

void qdictionnaire::AfficheEditeur()
{
    NbLignesEditeur=ui->textEditCodeEditor->blockCount();
}

void qdictionnaire::AfficheDefinition()
{
        if ( !  IndexDefinition.isEmpty())
        ChercherDefinition(IndexDefinition);
        else 	if ( ui->comboBoxHomographes->currentIndex() != -1 )
          {
            IndexDefinition=HomographesList.at(ui->comboBoxHomographes->currentIndex());
            ChercherDefinition(IndexDefinition);
          }

}

void qdictionnaire::AfficheCitation()
{
        if ( !ui->labelMotTrouveInfo->text().isEmpty() || !ui->lineEditMotCherche->text().isEmpty() ) {
        QString chainecherchee=ui->labelMotTrouveInfo->text();
        ChercherCitations(chainecherchee);
         }
        equalizeCitationsColumns();
}


void qdictionnaire::AskRime( QString mot)
{
    ui->lineEditMotCherche->setText(mot);
    AfficheTabulation(Onglet::Definition);
}


void qdictionnaire::AskDefinition( QString mot)
{
    ui->lineEditMotCherche->setText(mot);
    AfficheTabulation(Onglet::Definition);
}

void qdictionnaire::AskConjugaison( QString mot)
{
    ui->tabWidgetMode->addTab( conj.Panneau,QString::fromUtf8("Conjugaison")) ;
    OngletVerbe=true;
    QSqlQuery query=QSqlQuery("SELECT lemme FROM dico WHERE ( ortho = '"+ mot +"' AND cgram ='VER')" ) ;
    query.next();
    VerbeAConjuguer=query.value(0).toString();
    ui->lineEditMotCherche->setText(VerbeAConjuguer);
     AfficheTabulation(Onglet::Conjugaisons);
}

void qdictionnaire::AskCitations( QString mot)
{
    ui->lineEditMotCherche->setText(mot);
    AfficheTabulation(Onglet::Citation);
}
void qdictionnaire::AfficheConjugaison()
{
        VerbeAConjuguer=ui->labelRacine->text();
        QSqlQuery query("SELECT lemme FROM dico WHERE ( ortho = '"+ui->labelMotTrouveInfo->text()+"' AND cgram = 'VER' )");
        query.next() ;
        VerbeAConjuguer=query.value(0).toString();
        if ( VerbeAConjuguer.isNull() )
        {
             OngletVerbe=false;
             ui->tabWidgetMode->removeTab(Onglet::Conjugaisons);
        }
        else { conj.AfficheConjugaison(VerbeAConjuguer); }
}

 void qdictionnaire::MiseAJourHaut()
{
// en fait toutes les recherches vont se faire non pas a  partir du chanmp  ui->labelMotTrouve qui est le champ d'entree mais du champ ui->lineEditMotCherche qui est
// le resultat de la recherche orthographiqueui->comboBoxHomophones->setMinimumWidth(fontsize*10);
        QString mottrouve;
        QSqlQuery query;
        QString type;
        if ( ! ui->lineEditMotCherche->text().isEmpty() )
        {
                        // On cherche l'orthographe exacte si un mot est entre
                        query=QSqlQuery("SELECT ortho,PH,ID,cgram FROM dico WHERE ( ortho = \""+ ui->lineEditMotCherche->text() +"\")" ) ;
                        if ( query.next() )	{
                            // le mot est trouve dans l'orthographe exacte
                            mottrouve=query.value(0).toString(); PhonemeCherchee=query.value(1).toString() ;
                            qDebug()<<&IndexDefinition;
                            IndexDefinition=query.value(2).toString();type=query.value(3).toString();
                        }
                        else {
                            // le mot n'est pas trouve donc on cherche le phoneme
                                        phonemise mot;
                                        QString phoneme= mot.Text2Phoneme( ui->lineEditMotCherche->text());
                                        query=QSqlQuery("SELECT PH,ortho,ID,cgram FROM dico WHERE ( PH = \""+ phoneme +"\")" ) ;
                                        if ( query.next())  {
                                            // on trouve une phonème donc un mot qui peut remplacer
                                            mottrouve=query.value(1).toString();
                                            PhonemeCherchee=query.value(0).toString() ;
                                            IndexDefinition=query.value(2).toString();
                                            type=query.value(3).toString();
                                        }
                        }
         }
         else if (! IndexDefinition.isEmpty() )  {
                query=QSqlQuery("SELECT ortho,PH,cgram FROM dico WHERE ( ID = \""+ IndexDefinition +"\")" ) ;
                query.next();
                mottrouve=query.value(0).toString();
                PhonemeCherchee=query.value(1).toString();
                type=query.value(2).toString();
                ui->labelMotTrouveInfo->clear();
                ui->lineEditMotCherche->clear();
                }
        else if ( ! ui->lineEditMotCherche->text().isEmpty() ) {
                     query=QSqlQuery("SELECT ortho,PH,ID,cgram FROM dico WHERE ( ortho = \""+ ui->lineEditMotCherche->text() +"\")" ) ;
                     if ( query.next() )	{
                         mottrouve=query.value(0).toString();
                         PhonemeCherchee=query.value(1).toString() ;
                         IndexDefinition=query.value(2).toString();type=query.value(3).toString();}
                     else {
                           phonemise mot;
                           QString phoneme= mot.Text2Phoneme( ui->lineEditMotCherche->text());
                           query=QSqlQuery("SELECT PH,ortho,ID,cgram FROM dico WHERE ( ortho = \""+ phoneme +"\")" ) ;
                           if ( query.next())  {
                               mottrouve=query.value(0).toString();
                               PhonemeCherchee=query.value(1).toString() ;
                               IndexDefinition=query.value(2).toString();type=query.value(3).toString();}
                           }
                        }
        else if ( !ui->labelMotTrouveInfo->text().isEmpty() )
            mottrouve=ui->labelMotTrouveInfo->text() ;
        if (  mottrouve.isEmpty())
        {
            ;
        }
        else {
                EffacerChampsSaufRecherche();
                ConstruitListeHomographes(mottrouve,type);
                ConstruitListeHomophones(mottrouve,type);
                ui->labelMotTrouveInfo->setText(mottrouve);
        }
         ui->labelPhonemisationInfo->setText(PhonemeCherchee);
         ui->lineEditMotCherche->clear();
        qWarning()<<IndexDefinition;

  }



void qdictionnaire::FabriquePhrase ()
{
        setCursor (Qt::WaitCursor );
        ui->tabWidgetMode->setCurrentIndex (Onglet::Edition);
        QString mot;
        mot=ui->lineEditMotCherche->text() ;
        if ( mot == "" )
         mot=ui->labelMotTrouveInfo->text() ;
        MiseAJourHaut();
        oulipo phrase ;
        QString ph=phrase.ComposePhrase(mot,ui->spinBoxRichessRime->value(),ui->comboBoxTypePhrase->currentText());
        ui->textEditCodeEditor->textCursor().insertText(ph+"\n");
        unsetCursor();

}

// appele quand on change le type d'une rime, il faut donc forcer l'onglet rime et appeler l'affichage
void qdictionnaire::AfficherRimeParType()
{
        ui->tabWidgetMode->setCurrentIndex (Onglet::Definition ) ;
        AfficheTabulation();
}

// Appelee quand on modifie la profondeur de la rime ( donc on force l'affichage de l'onglet rime)
void qdictionnaire::AfficherRimeParNbPhoneme ()
{
        ui->tabWidgetMode-> setCurrentIndex(Onglet::Definition);
        AfficheTabulation();
}


void qdictionnaire::ChercherRimes( QString phoneme, int nbphoneme )
{
        // rime est ainsi la valeur que nous allons devoir chercher dans la base
        debug::message(3,QString("nbphoneme %s").arg(nbphoneme));
        QString Where;
        wherecondition WC ;
        Where=WC.WhereCondition(phoneme,ui->comboBoxTypeRime->currentText(),nbphoneme);
        QString requete="SELECT ortho,nbsyll,genre,cgram,lemme,PH,freq,ID FROM dico WHERE ( "+Where+" )";
        debug::message(2,"Req5 ortho, nbsyl: " + requete);
        ui->tableViewListeReponse->setShowGrid (false );
        ui->tableViewListeReponse->setSortingEnabled ( true );
        // 	 QHeaderView * Header = ListeReponse->horizontalHeader() ;
        m_rimeModel->Phoneme=phoneme;
        m_rimeModel->setQuery(requete);
        m_rimeProxyModel->setSourceModel(m_rimeModel);
        ui->tableViewListeReponse->setModel(m_rimeProxyModel);
        equalizeReponsesColumns();
        connect (ui->tableViewListeReponse, SIGNAL(clicked(QModelIndex)), this, SLOT (DefinitionMot()));

        // 	 ListeReponse->resizeColumnToContents ( 6) ;
        QString count;
        NbRimes=m_rimeModel->rowCount();
        if ( m_rimeModel->rowCount() == 256 ) { count =QString::fromUtf8("plus de 256 trouvées"); }
        else if ( m_rimeModel->rowCount() <= 1 ) {   count = QString::number(m_rimeModel->rowCount()) ; count+=QString::fromUtf8(" trouvée") ; }
        else { count = QString::number(m_rimeModel->rowCount()) ; count+=QString::fromUtf8(" trouvées") ; }
        ui->labelNombreMotTrouveInfo->setText(count );
}



void qdictionnaire::ChercherDefinition( QString index )
{
   formatage F;
   QString requete;
   QString cgram;
   QString ortho;
   QString racine;
   QString genre;
   QString nombre;
   //chainecherchee.toLower(); // en minuscule dans la base
   requete="SELECT lemme,genre,nbsyll,cgram,ortho,nombre FROM dico WHERE ( ID = \""+index+"\")";
   setCursor (Qt::WaitCursor );
   QSqlQuery query(requete);
   debug::message(2,"Recherche typée: requete :"+requete);
   QString definitions,etymologies , synonymes, antonymes,expressions,approchants;
   query.next() ;
   racine=query.value(0).toString();
   ui->labelRacine-> setText(racine);
   genre=query.value(1).toString();
   ui->labelGenre->setText(F.FormateGenre(genre));
   ui->labelSyllabes->setText(F.FormateSyllabes(query.value(2).toString()));
   cgram=query.value(3).toString();
   ui->labelType->setText(cgram);
   ortho=query.value(4).toString();
   ui->labelMotTrouveInfo->setText(ortho);
   nombre=query.value(5).toString();
   QString requete2;
   requete2=QString("SELECT definition FROM definitions WHERE ( ortho = '")+ui->labelMotTrouveInfo->text()+QString("')");
   QSqlQuery query2(requete2);
   query2.next();
   definitions = query2.value(0).toString();
   bool definitionmise=false;
   if ( ui->labelType->text() !=  "VER"  )
   {
       if ( OngletVerbe )
       {
           ui->tabWidgetMode->removeTab(Onglet::Conjugaisons);OngletVerbe=false;
       }
   }
   else
   {
       if ( !OngletVerbe )
        {
         ui->tabWidgetMode->addTab( conj.Panneau,QString::fromUtf8("Conjugaison")) ;
         OngletVerbe=true;
         VerbeAConjuguer=ui->labelRacine->text();
        }
       else
        {
         VerbeAConjuguer=ui->labelRacine->text();
        }
       QString requete2;
       requete2=QString("SELECT definition FROM definitions WHERE ( ortho = '"+VerbeAConjuguer+"')");
       QSqlQuery query2(requete2);
       query2.next();
       definitions = query2.value(0).toString();
       ui->textBrowserDefinition->setText(
       F.FormateDefinition(VerbeAConjuguer,
                definitions,conjugaison::AfficheDefinition(VerbeAConjuguer,ui->labelMotTrouveInfo->text()) ));
       //

      definitionmise=true;
   }
  if ( ! definitionmise)
 ui->textBrowserDefinition->setText(F.FormateDefinition (ui->labelMotTrouveInfo->text(),definitions));

  //connect ( DefinitionText, SIGNAL(doubleClicked()), this, SLOT (TestSelection()));
  ui->textBrowserEtymologie->setText(F.FormateEtymologie(F.getEtymologie()));
  ui->textBrowserExpressions->setText(F.FormateExpression(F.getExpression()));
  ConstruitListeSynonymes(ortho,cgram,racine,genre,nombre);
  ui->comboBoxAntonyme->clear();
  ui->comboBoxAntonyme->addItems(F.FormateListe(F.getAntonyme()));
  ConstruitListeAnagrammes(ui->labelMotTrouveInfo->text());
  //IndexDefinition.clear();
  unsetCursor();
}

void qdictionnaire::ConstruitListeSynonymes( QString ortho,QString cgram,QString racine,QString genre,QString nombre)
{
   ui->comboBoxSynonyme->clear();
   synonymes syn;
   ui->comboBoxSynonyme->addItems(syn.ListeSynonymes(ortho,cgram,racine,genre,nombre));
}


void  qdictionnaire::ChercherCitations ( QString chainecherchee)
{
        if ( ! chainecherchee.isEmpty()) {
  chainecherchee=chainecherchee.toLower(); // en minuscule dans la base
  QString  requete=QString::fromLatin1("SELECT genre,rubrique,citation,auteur FROM citations WHERE citation LIKE '")
                 + chainecherchee
                 + QString::fromLatin1(" %%' OR citation LIKE '%% ")
                 + chainecherchee
                 + QString::fromLatin1("' OR citation LIKE '%% ")
                 + chainecherchee
                 + QString::fromLatin1(" %%'");
  debug::message(2,"Req6 citation  : " + requete);
  ui->tableViewCitations->setShowGrid (false );
  ui->tableViewCitations->setSortingEnabled ( true );
  ui->tableViewCitations->setSelectionBehavior ( QAbstractItemView::SelectItems );
  ui->tableViewCitations->setSelectionMode( QAbstractItemView::SingleSelection );

  QSqlQueryModel *model2 = new QSqlQueryModel;
  QSortFilterProxyModel *CitationModel = new QSortFilterProxyModel(this);
  CitationModel->setSourceModel(model2);
  model2->setQuery(requete);
  ui->tableViewCitations->setModel(CitationModel);
  if ( ! ConnecteursCitation) connect ( ui->tableViewCitations, SIGNAL(pressed(QModelIndex)), this, SLOT (AfficheUneCitation( QModelIndex)));
  ConnecteursCitation=true;

  NbCitations=CitationModel->rowCount();
  for ( int i =0 ; i<256; i++) { ui->tableViewCitations->setRowHeight ( i, 80 );}
}
}


void qdictionnaire::ActualiseCitation ( int value)
{
        formatage F;
        CitationUi.citation->setText(
                        F.FormateCitation(
                                        ui->labelMotTrouveInfo->text(),
                                        ListeIndexCitation.sibling(value,2).data().toString(),
                                        ListeIndexCitation.sibling(value,3).data().toString(),
                                        ListeIndexCitation.sibling(value,1).data().toString()
                                        )
                        );
        CitationUi.navigation->setSliderPosition(value);
}

// Affiche une citation dans avec un ascenseur dans une fenetre dialogue
void qdictionnaire::AfficheUneCitation( QModelIndex index)
{
          formatage F;
          winCitation = new QDialog;
          CitationUi.setupUi(winCitation);
          winCitation->setModal(true);
          winCitation->show();

          CitationUi.navigation->setMinimum(0);
          CitationUi.navigation->setMaximum(NbCitations-1);
          ListeIndexCitation=index;
          CitationUi.citation->setText(
                          F.FormateCitation(
                                          ui->labelMotTrouveInfo->text(),
                                          ListeIndexCitation.sibling(ListeIndexCitation.row(),2).data().toString(),
                                          ListeIndexCitation.sibling(ListeIndexCitation.row(),3).data().toString(),
                                          ListeIndexCitation.sibling(ListeIndexCitation.row(),1).data().toString()
                                          )
                          );

          connect ( CitationUi.navigation,SIGNAL(valueChanged(int)),this,SLOT(ActualiseCitation(int)));
          connect ( CitationUi.FermerBouton,SIGNAL(clicked()),this,SLOT(closeCitation()));
  }

// Ferme la fenÃªtre de citation
void qdictionnaire::closeCitation ()
{
          winCitation->close();
}

//Gère la recherche libre des citation par auteur
void qdictionnaire::ChercherLibreCitationAuteur()
{
  QString chainecherchee = ui->lineEditRechercheLibreCitation->text();
  chainecherchee= chainecherchee.toLower();

  QString requete=QString::fromLatin1("SELECT genre,rubrique,citation,auteur FROM citations WHERE auteur LIKE '")
                                 + chainecherchee
                                 + QString::fromLatin1("%%' OR auteur LIKE '%%")
                                 + chainecherchee
                                 + QString::fromLatin1("' OR auteur LIKE '%%")
                                 + chainecherchee
                                 + QString::fromLatin1("%%'");
  debug::message(2,"Reqlibre citation  : " + requete);
  ui->tableViewCitations->setShowGrid (false );
  ui->tableViewCitations->setSortingEnabled ( true );
  QSqlQueryModel *model2 = new QSqlQueryModel;
  QSortFilterProxyModel *CitationModel = new QSortFilterProxyModel(this);
  //CitationModel->Mot=chainecherchee;
  CitationModel->setSourceModel(model2);
  model2->setQuery(requete);
  ui->tableViewCitations->setModel(CitationModel);
   if ( ! ConnecteursCitation) connect (ui->tableViewCitations, SIGNAL(pressed(QModelIndex)), this, SLOT (AfficheUneCitation( QModelIndex)));
  ConnecteursCitation=true;
  NbCitations=CitationModel->rowCount();
  for ( int i =0 ; i<256; i++) {ui->tableViewCitations->setRowHeight ( i, 80 );}
  equalizeCitationsColumns();
}


void qdictionnaire::ChangerMode ()
{
// mMode=ui->tabWidgetMode->currentIndex();
// LancerRecherche();
}


// SLot appele lors de l'actionnement de la recherche homonymique
void qdictionnaire::RelancerRechercheHomographes()
{
        ui->comboBoxHomographes->disconnect();

        if ( ! ui->comboBoxHomographes->currentText().isEmpty() ){
                IndexDefinition=HomographesList.at(ui->comboBoxHomographes->currentIndex());
                ui->labelMotTrouveInfo->clear();
                if ( ui->tabWidgetMode->currentIndex()==Onglet::Definition) AfficheTabulation() ;
                else ui->tabWidgetMode->setCurrentIndex(Onglet::Definition) ;

        }
}
void qdictionnaire::RelancerRechercheHomophones()
{
        ui->comboBoxHomophones->disconnect();
        if ( ! ui->comboBoxHomophones->currentText().isEmpty() ){
                IndexDefinition=HomophonesList.at(ui->comboBoxHomophones->currentIndex());
                ui->labelMotTrouveInfo->clear();
                if ( ui->tabWidgetMode->currentIndex()==Onglet::Definition) AfficheTabulation()  ;
                else ui->tabWidgetMode->setCurrentIndex(Onglet::Definition) ;
        }
}



void qdictionnaire::RelancerRechercheAnagramme()
{
        ui->comboBoxAnagrammes->disconnect();
        if ( !ui->comboBoxAnagrammes->currentText().isEmpty() ){
        IndexDefinition=AnagrammesList.at(ui->comboBoxAnagrammes->currentIndex());
        AfficheTabulation();

        }
}


 void qdictionnaire::EffacerChampsSaufRecherche()
{
  ui->labelNombreMotTrouveInfo->setText(QString::fromUtf8("0 trouvée"));

  QSqlQueryModel *model = new QSqlQueryModel;
  ui->tableViewListeReponse->setModel(model);
  ui->comboBoxAnagrammes->clear();
  ui->comboBoxHomographes->clear();
  ui->comboBoxHomophones->clear();
  ui->textBrowserDefinition->clear();
  ui->textBrowserEtymologie->clear();
  ui->textBrowserExpressions->clear();
  ui->labelGenre->clear();
  ui->labelSyllabes->clear();
  ui->labelRacine->clear();
  ui->labelType->clear();
  QSqlQueryModel *model2 = new QSqlQueryModel;
  ui->tableViewCitations->setModel(model2);
  HomophonesList.clear();
  HomographesList.clear();
 // IndexDefinition.clear();
}

void qdictionnaire::EffacerBouton_clicked ()
{
  EffacerChampsSaufRecherche();
  if ( OngletVerbe ) {ui->tabWidgetMode->removeTab(Onglet::Conjugaisons);OngletVerbe=false;}
  ui->labelMotTrouveInfo->clear();
  ui->labelNombreMotTrouveInfo->clear();
  ui->lineEditMotCherche->clear();
}


void qdictionnaire::configurationdef()
{

         m_configuration->MotCherche=ui->labelMotTrouveInfo->text();
         m_configuration->setTranslator(m_systemtranslator,m_appltranslator);
         m_configuration->show();

}

void qdictionnaire::ChangeLangs(QTranslator *systr,QTranslator *apptr)
{
    Lanquage l;
    qApp->removeTranslator(m_appltranslator);
    qApp->removeTranslator(m_systemtranslator);
    m_appltranslator=apptr;
    m_systemtranslator=systr;
    qApp->installTranslator(m_appltranslator);
    qApp->installTranslator(m_systemtranslator);
    ui->retranslateUi(this);

}

void qdictionnaire::ReloadConfig()
{
    QSettings settings;
    QString style=settings.value("Style").toString();
    feuillestyle feuille(style);
    m_application->setStyleSheet(feuille.GetStyle());
    m_application->setFont(QFont(settings.value("PoliceDefaut").toString(),settings.value("TaillePoliceDefaut").toInt()));
    ui->textEditCodeEditor->ReConfigure();
    ui->tabWidgetMode->setCurrentIndex(settings.value("Mode").toInt());
    AfficheTabulation();
    ui->spinBoxRichessRime->setValue(settings.value("RichesseRimeDefaut").toInt());
    if ( settings.value("AfficherLieux")==1) ui->checkBoxLieux->setCheckState(Qt::Checked);
    else ui->checkBoxLieux->setCheckState(Qt::Unchecked);
    if ( settings.value("AfficherPrenoms")==1) ui->checkBoxPrenoms->setCheckState(Qt::Checked);
    else ui->checkBoxPrenoms->setCheckState(Qt::Unchecked);
    if ( settings.value("AfficherVerbes")==1) ui->checkBoxVerbes->setCheckState(Qt::Checked);
    else ui->checkBoxVerbes->setCheckState(Qt::Unchecked);
    if ( settings.value("AfficherAdjectifs")==1) ui->checkBoxAdjectifs->setCheckState(Qt::Checked);
    else ui->checkBoxAdjectifs->setCheckState(Qt::Unchecked);
    if ( settings.value("AfficherNoms")==1) ui->checkBoxNoms->setCheckState(Qt::Checked);
    else ui->checkBoxNoms->setCheckState(Qt::Unchecked);
    if ( settings.value("AfficherSigles")==1) ui->checkBoxSigles->setCheckState(Qt::Checked);
    else ui->checkBoxSigles->setCheckState(Qt::Unchecked);



}

void qdictionnaire::apropos()
{
  Ui::apropos ui;
  ui.setupUi(m_apropos);
  m_apropos->show();
  QString VLong=QString("Version ")+VERSIONWORK;
  QString DateBuild=DATEBUILD;
  DateBuild=tr("Générée le ")+DateBuild;
  ui.APVersion->setText(VLong);
  ui.APDate->setText(DateBuild);
  QString Platform= QT_VERSION_STR;
#if defined( Q_OS_LINUX)
  Platform=QString("Qt Linux")+Platform;
#endif
#if defined( Q_OS_WIN32)
  Platform=QString::fromUtf8("Qt Windows")+Platform;
#endif
#if defined( Q_OS_MAC)
 Platform=QString::fromUtf8("MacOSX Qt ")+Platform;
#endif
  ui.APPlateforme->setText(Platform);
}

void qdictionnaire::aide()
{
  QDialog *window = new QDialog (this);
  Ui::aide ui;
  ui.setupUi(window);
  window->setModal(true);
  window->show();
}

void qdictionnaire::impression()
{
        windowPrint = new QDialog;
        PrintUI.setupUi(windowPrint);
        windowPrint->setModal(true);
        windowPrint->show();
        bool NothingToPrint=true;
        PrintUI.ImprimerBouton->hide();
        PrintUI.PreviewBouton->hide();

        if ( ui->textBrowserDefinition->document()->isEmpty()) { PrintUI.definition->setDisabled(true) ;} else { NothingToPrint=false;}
        if ( ui->textBrowserExpressions->document()->isEmpty()) { PrintUI.expression->setDisabled(true) ;} else { NothingToPrint=false;}
        if ( ui->textBrowserEtymologie->document()->isEmpty()) { PrintUI.etymologie->setDisabled(true) ;} else { NothingToPrint=false;}
        if ( ui->comboBoxAnagrammes->count() == 0 )  { PrintUI.anagramme->setDisabled(true) ;} else { NothingToPrint=false;}
        if ( ui->comboBoxHomophones->count() == 0 )  { PrintUI.homonyme->setDisabled(true) ;} else { NothingToPrint=false;}
        if ( NbCitations == 0  ) { PrintUI.citation->setDisabled(true) ;} else { NothingToPrint=false;}
        if ( NbRimes == 0  ) { PrintUI.rime->setDisabled(true) ;} else { NothingToPrint=false;}
        if ( NbLignesEditeur == 0 ){ PrintUI.editeur->setDisabled(true) ;} else { NothingToPrint=false;}

        if ( NothingToPrint ) {
                        PrintUI.tous->setDisabled(true) ;
                        } else {

                connect ( PrintUI.ImprimerBouton,SIGNAL(clicked()),this,SLOT(Print()));
                connect ( PrintUI.PreviewBouton,SIGNAL(clicked()),this,SLOT(Preview()));
                connect ( PrintUI.tous, SIGNAL (stateChanged(int)),this,SLOT(AllSelectDeselect()));
        }
        connect ( PrintUI.AnnulerBouton, SIGNAL (clicked()),this,SLOT(ClosePrint())) ;

        connect (PrintUI.definition,SIGNAL(stateChanged(int)),this, SLOT(EnablePrinter()));
        connect (PrintUI.etymologie,SIGNAL(stateChanged(int)),this, SLOT(EnablePrinter()));
        connect (PrintUI.expression,SIGNAL(stateChanged(int)),this, SLOT(EnablePrinter()));
        connect (PrintUI.homonyme,SIGNAL(stateChanged(int)),this, SLOT(EnablePrinter()));
        connect (PrintUI.anagramme,SIGNAL(stateChanged(int)),this, SLOT(EnablePrinter()));
        connect (PrintUI.rime,SIGNAL(stateChanged(int)),this, SLOT(EnablePrinter()));
        connect (PrintUI.citation,SIGNAL(stateChanged(int)),this, SLOT(EnablePrinter()));
        connect (PrintUI.editeur,SIGNAL(stateChanged(int)),this,SLOT(EnablePrinter()));
}


void qdictionnaire::EnablePrinter()
{
        if ( PrintUI.definition->checkState () == Qt::Unchecked &&
                 PrintUI.etymologie->checkState () == Qt::Unchecked &&
                 PrintUI.expression->checkState () == Qt::Unchecked &&
                 PrintUI.homonyme->checkState () == Qt::Unchecked &&
                 PrintUI.anagramme->checkState () == Qt::Unchecked &&
                 PrintUI.rime->checkState () == Qt::Unchecked &&
                 PrintUI.editeur->checkState() == Qt::Unchecked &&
                 PrintUI.citation->checkState () == Qt::Unchecked ){
                 PrintUI.ImprimerBouton->hide() ;
                 PrintUI.PreviewBouton->hide() ;

   } else {
                 PrintUI.ImprimerBouton->show();
                 PrintUI.PreviewBouton->show();


   }
}
void qdictionnaire::ClosePrint ()
{
                windowPrint->close();
}
void qdictionnaire::AllSelectDeselect ()
{
                if (  PrintUI.tous->isChecked() )
         {
                   if (  PrintUI.definition->isEnabled()) {PrintUI.definition->setCheckState(Qt::Checked); }
                   if (  PrintUI.etymologie->isEnabled()) {PrintUI.etymologie->setCheckState(Qt::Checked);}
                   if (  PrintUI.expression->isEnabled()) {PrintUI.expression->setCheckState(Qt::Checked);}
                   if (  PrintUI.homonyme->isEnabled()) {PrintUI.homonyme->setCheckState(Qt::Checked);}
                   if (  PrintUI.anagramme->isEnabled()) {PrintUI.anagramme->setCheckState(Qt::Checked);}
                   if (  PrintUI.rime->isEnabled()) {PrintUI.rime->setCheckState(Qt::Checked);}
                   if (  PrintUI.citation->isEnabled()) {PrintUI.citation->setCheckState(Qt::Checked);}
                   if ( PrintUI.editeur->isEnabled()) {PrintUI.editeur->setCheckState(Qt::Checked);}
         }
        else
        {
                   PrintUI.citation->setCheckState(Qt::Unchecked);
                   PrintUI.rime->setCheckState(Qt::Unchecked);
                   PrintUI.definition->setCheckState(Qt::Unchecked);
                   PrintUI.etymologie->setCheckState(Qt::Unchecked);
                   PrintUI.expression->setCheckState(Qt::Unchecked);
                   PrintUI.homonyme->setCheckState(Qt::Unchecked);
                   PrintUI.anagramme->setCheckState(Qt::Unchecked);
                   PrintUI.editeur->setCheckState(Qt::Unchecked);
        }
}

void qdictionnaire::Preview()
{

        QPrinter printer(QPrinter::HighResolution);
        printer.setFullPage(true);
        QPrintPreviewDialog *dialogue = new QPrintPreviewDialog(&printer, this);

        connect( dialogue, SIGNAL(paintRequested(QPrinter *)), this, SLOT(producePrint(QPrinter *)) );
        dialogue->exec() ;

}

void qdictionnaire::Print()
{
         QPrinter printer(QPrinter::HighResolution);
        printer.setFullPage(true);
        QPrintDialog *dialogue = new QPrintDialog(&printer, this);
        dialogue->addEnabledOption(QPrintDialog::PrintSelection);
        if (dialogue->exec() == QDialog::Accepted) {
           producePrint(&printer);
           delete dialogue;
           windowPrint->close();
 }
 }

void qdictionnaire::producePrint ( QPrinter * printer)
{
        QString page;
        page=Stylise("","Start");
        page+=Stylise(ui->labelMotTrouveInfo->text(),"T");

                if ( PrintUI.definition->checkState() == Qt::Checked){
                        page+=Stylise("D&eacute;finition","T1");
                        page+=ui->textBrowserDefinition->toHtml();
                }
                if ( PrintUI.etymologie->checkState() ==Qt::Checked) {
                        page+=Stylise("Etymologie","T1");
                        page+=ui->textBrowserEtymologie->toHtml();
                }
                if ( PrintUI.expression->checkState() ==Qt::Checked) {
                        page+=Stylise("Expressions","T1");
                        page+=ui->textBrowserExpressions->toHtml();
                }
                if ( PrintUI.homonyme->checkState() ==Qt::Checked) {
                        QStringList liste;
                        int i;
                        for (i=0;i<ui->comboBoxHomophones->count();i++) {liste+=ui->comboBoxHomophones->itemText(i);}
                        QString s=(i==1)?"":"s";
                        page+=Stylise("Homonyme"+s,"T1");
                        QString l=liste.join(", ");
                        page+=l;
                }if ( PrintUI.anagramme->checkState() ==Qt::Checked) {
                        QStringList liste;
                        int i;
                        for (i=0;i<ui->comboBoxAnagrammes->count();i++) {liste+=ui->comboBoxAnagrammes->itemText (i);}
                        QString s=(i==1)?"":"s";
                        page+=Stylise("Anagramme"+s,"T1");
                        QString l=liste.join(", ");
                        page+=l;
                }
                if ( PrintUI.citation->checkState() == Qt::Checked) {
                        page+=Stylise("Citations","T1");
                        for ( int i=0;i<ui->tableViewCitations->model()->rowCount(); i++) {
                                QString citation=ui->tableViewCitations->model()->sibling(i,2, QModelIndex() ).data().toString();
                                QString auteur=ui->tableViewCitations->model()->sibling(i,3, QModelIndex() ).data().toString();
                                page+=Stylise(citation,"C")+Stylise(auteur,"A");
                        }
                }
                if ( PrintUI.rime->checkState() == Qt::Checked) {
                        page+=Stylise("Rimes","T1");
                        QString rimes[8];
                        QString phonetiquemot=ui->labelPhonemisationInfo->text();
                        for ( int i=0;i<ui->tableViewListeReponse->model()->rowCount(); i++) {
                                QString rime=ui->tableViewListeReponse->model()->sibling(i,0, QModelIndex() ).data().toString();
                                QString phonetique=ui->tableViewListeReponse->model()->sibling(i,5, QModelIndex() ).data().toString();
                                rimes[NbRimeCommun(phonetiquemot,phonetique)]+=", "+rime;
                        }
                        for (int i=0;i<8;i++ ) {
                                if ( rimes[i] != "" ) {
                                        rimes[i].replace(QRegExp("^,")," ");
                                        QString num;
                                        page+=Stylise("Profondeur "+num.setNum(i-1),"T2")+rimes[i]+"<p>";
                                }
                        }
                }
                if ( PrintUI.editeur->checkState() == Qt::Checked)
                {
                    page+=Stylise("Edition","T1");
                    page+=Stylise(ui->textEditCodeEditor->toPlainText(),"PT");
                }
                page+=Stylise("","End");
                QTextDocument Texte ;
                Texte.setHtml(page);
                Texte.print(printer) ;
}

int qdictionnaire::NbRimeCommun(QString phonetiquemot,QString phonetique)
{
        int nb=1;
        while (phonetiquemot.at(phonetiquemot.length()-nb) == phonetique.at(phonetique.length()-nb)) {nb++;}
        return (nb);
}

QString qdictionnaire::Stylise ( QString chaine, QString Type )
{   if ( Type == "Start") { return ("<html><body>");}
        if ( Type == "End")   { return ("</body></html>"); }
        if ( Type =="C" ) { return ("<P>&diams;<span Style=\"Font: 13px Verdana;\">"+chaine);}
        if ( Type =="A" ) { return ("<bold><i> ["+chaine+"]</i></bold></span></p>");}
        if ( Type =="T" ) { return ("<P><span Style=\"Font: 24px Verdana; color:white;background-color:rgb(110,50,50)\"> "+chaine+" </span></P>" );}
        if ( Type =="T1") {	return ("<P><span Style=\"Font: 18px Verdana; color:gray;\">"+chaine+"</span></P>" );}
        if ( Type =="T2") {	return ("<span Style=\"Font: 16px Verdana; color:gray;\">&diams; "+chaine+"</span> : " );}
        if ( Type == "PT") {
        QStringList t=chaine.split("\n");
        QString c= t.join ("<br>");
        return ( c);
        }


        return (chaine);
}


void qdictionnaire::AffichagePhonemisation()
{
     phonemisation *phonwin = new phonemisation()  ;

     phonwin->textEdit=ui->textEditCodeEditor->toPlainText();
     phonwin->Lance();
}

void qdictionnaire::Statistiques()
{
  m_statistiques =new QDialog(this);
  Ui::Statistiques ui;
  ui.setupUi(m_statistiques);
  m_statistiques->show();
  QSqlQuery query("select count(*) from citations");
  query.next() ;
  ui.NbCitations->setText(QLocale::system().toString(query.value(0).toInt()));
  query.prepare("select count(*) from dico");
  query.exec();
  query.next() ;
  ui.NBPhonemise->setText(QLocale::system().toString(query.value(0).toInt()));
  query.prepare("select count(*) from definitions");
  query.exec();
  query.next() ;
  ui.NbDefinitions->setText(QLocale::system().toString(query.value(0).toInt()));
  QFileInfo file(mPathBase) ;
  qint64 taille=file.size();
  taille=(int)(taille/1000000);
  QString t;
  t.setNum(taille);
  ui.TailleBase->setText(t);


}

QString qdictionnaire::PartieRimeChaine ( QString chaine, QString phoneme )
{
// renvoi la partie qui rime de la chaine : ex : PartieRimeChaine("poupee","pe") => pee
QString retour;
QString ch ;
phonemise mot;
for ( int i = chaine.length() ; i > 0 ; i-- ) {
 ch=chaine;
 ch.remove(i,chaine.length()-i+1);
 if ( mot.Text2Phoneme(ch) == phoneme ) { retour=ch;}
 }
 return (retour);
}

// ets le slot appele quand on clique dans la liste des rimes
void qdictionnaire::DefinitionMot()
{
//  int Col=ui->tableViewListeReponse->currentIndex().column();
  QString ph=ui->tableViewListeReponse->currentIndex().sibling(ui->tableViewListeReponse->currentIndex().row(),5).data().toString() ;
  QString motapparente=ui->tableViewListeReponse->currentIndex().sibling(ui->tableViewListeReponse->currentIndex().row(),4).data().toString() ;
  QString index=ui->tableViewListeReponse->currentIndex().sibling(ui->tableViewListeReponse->currentIndex().row(),7).data().toString() ;
  QString type=ui->tableViewListeReponse->currentIndex().sibling(ui->tableViewListeReponse->currentIndex().row(),3).data().toString() ;
  if (ui->tableViewListeReponse->currentIndex().row()!=-1 )
  {
      QModelIndexList rows = ui->tableViewListeReponse->selectionModel()->selectedRows();
      m_LigneRimeCourante=ui->tableViewListeReponse->currentIndex().row();
      m_ModelRimeCourante =rows.first().model()->index(rows.first().row()-1,rows.first().column());
  }

//  if ( Col == Conjugaisons || Col== InfoRime)  {
//        IndexDefinition=index;
//        ui->labelMotTrouveInfo->clear();
//        ui->tabWidgetMode-> setCurrentIndex(Onglet::Definition);
//  }
}



QString qdictionnaire::InverseMot ( QString mot ) {
  QString m;
  for (int i=0;i<mot.length();i++){
        m[i]=mot[mot.length()-i-1];
  }
  return (m);
}

void qdictionnaire::setTextLineEditCherche(QString texte)
{
    ui->lineEditMotCherche->setText(texte);
}

void qdictionnaire::setTranslator(QTranslator *apptranslator, QTranslator *systemtranslator)
{
    m_systemtranslator=systemtranslator;
    m_appltranslator=apptranslator;
}



QString  qdictionnaire::WhereAnagramme (QString chainecherchee)
{
        // Cette fonction est rapide mais elle demande d'epurer les extra caracteres :
        // papa va provoquer une requete ( ortho like '%p%' and ortho like "%a"
        // et ce pour des mots de  longueur 4 on va donc recuperer pays il suffira de tester la presence ExtraCaracter
        QString where;
        int longueur=chainecherchee.length();
        QStringList tabcar= chainecherchee.split((QRegExp("")));
        foreach ( QString l , tabcar ) { debug::message (5,"tabcar : "+l);}
        QStringList list;
        tabcar.removeDuplicates ();
        for ( int i =1;  i<tabcar.size();i++)
        {
                QString val=QString::fromLatin1("ortho LIKE '%%")+tabcar.at(i)+QString::fromLatin1("%%'");
                 list.append(val);
        }
        where =list.join(" AND ");

        where=where+" AND length(ortho) = "+QString::number(longueur);

        return (where);
}

bool qdictionnaire::ExtraCaractere( QString chainetrouvee, QString chainecherchee)
{
        // y a t'il des caracteres en trop quand les mots ont des repetitions de lettres
         QStringList listecar= chainecherchee.split((QRegExp("")));
         for ( int i =1;  i<listecar.size();i++) {
                 chainetrouvee.remove(listecar.at(i));
         }
         return ( chainetrouvee.isEmpty() );
}

void qdictionnaire::TestSelection ()
{
        QTextCursor cursor(ui->textBrowserDefinition->textCursor());
                cursor.select(QTextCursor::WordUnderCursor);
        debug::message (4,"Text selected " + cursor.selectedText());
}


void qdictionnaire::AjouteListeFichier ( QString file)
{
         QSettings settings;
         int NB=settings.value("NombreFichiersListes").toInt();
         int NBFL;
         if ( NB < 10 ) NBFL=NB+1; else NBFL=NB;
         QStringList fichiers;
         int j=2;
         fichiers.append(file);
         for ( int i=1;i<=NB;i++)
         {
                 QString val=settings.value("FichiersListes"+QString::number(i)).toString();
                 if ( val != file )  {fichiers.append(val); j++;}
                 else { NBFL--; }
         }
         j=1;

         foreach ( QString f, fichiers) {
                 settings.setValue("FichiersListes"+QString::number(j),f);
                 j++;
         }
         settings.setValue("NombreFichiersListes",NBFL);
         AfficheMenuListeFichier();
}


void qdictionnaire::AfficheMenuListeFichier()
{
         ui->menuRecemmentOuverts->clear();
         QSettings settings;
         int NB=settings.value("NombreFichiersListes").toInt();
         for ( int i=1;i<=NB;i++)
         {
                 QAction *action=new QAction(this);
                 action->setText(settings.value("FichiersListes"+QString::number(i)).toString());
                 action->setObjectName (settings.value("FichiersListes"+QString::number(i)).toString()  );
                 ui->menuRecemmentOuverts->addAction(action);
                 connect ( action, SIGNAL ( triggered() ), this , SLOT ( OuvrirFichierAction() ));

         }
}

void qdictionnaire::AskandSave()
{
    if ( EditChange )
    {
    QMessageBox msgBox;
    msgBox.setText(tr("Le document n'a pas été sauvegardé"));
    msgBox.setInformativeText(tr("Voulez-vous sauvegarder les changements?"));
    msgBox.setStandardButtons(QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
    msgBox.setDefaultButton(QMessageBox::Save);
    int ret = msgBox.exec();
    switch (ret) {
            case QMessageBox::Save:
                    SauverFichier() ;
                    break;
            case QMessageBox::Discard:
                    break;
            case QMessageBox::Cancel:
                    return;
                    break;
             default:
                    break;
            }
    }
}

void qdictionnaire::OuvrirFichierAction()
{
  QObject *caller = QObject::sender () ;
  AskandSave();
  fichiertexte=caller->objectName();
  QFile file(fichiertexte);
  if (!file.open(QIODevice::ReadWrite | QIODevice::Text))    return;
  setCursor (Qt::WaitCursor );
  ui->textEditCodeEditor->SetPlainText( QString::fromUtf8(file.readAll ()));
  unsetCursor();
  ui->tabWidgetMode->setCurrentIndex(Onglet::Edition);
  EditChange=false;

}

void qdictionnaire::OuvrirDernierFichier()
{
    QSettings s;
    int i=1;
    while (s.value(QString("FichiersListes%1").arg(i)).isValid())
    {
        i++;
    }
    i--;
    if (i!=0)
    fichiertexte=s.value(QString("FichiersListes%1").arg(i)).toString();
    QFile file(fichiertexte);
    if (!file.open(QIODevice::ReadWrite | QIODevice::Text))
                   return;
    ui->textEditCodeEditor->SetPlainText( QString::fromUtf8(file.readAll ()));

    ui->tabWidgetMode->setCurrentIndex(Onglet::Edition);
    EditChange=false;
}

void qdictionnaire::OuvrirFichier()
{
  AskandSave();
  QSettings settings;
  QString DernierRep=settings.value("DernierRep").toString();
  if ( DernierRep == "" ) DernierRep=".";
  QFileDialog FileDialog;
  fichiertexte =FileDialog.getOpenFileName(this, "Ouvrir le fichier",DernierRep,"Fichiers textes(*)");
  QFileInfo fichier(fichiertexte);
  DernierRep= fichier.path();
  settings.setValue("DernierRep", DernierRep);
  QFile file(fichiertexte);
  if (!file.open(QIODevice::ReadWrite | QIODevice::Text))
                 return;
  setCursor (Qt::WaitCursor );
  ui->textEditCodeEditor->SetPlainText( QString::fromUtf8(file.readAll ()));
//  CorrectionOrthographique cor;
//  cor.Corrige(textEdit->document());
  unsetCursor();

  AjouteListeFichier(fichiertexte);
  ui->tabWidgetMode->setCurrentIndex(Onglet::Edition);
  EditChange=false;
}

void qdictionnaire::Correction()
{
    mCorrecteur->Corrige(ui->textEditCodeEditor->document());
}

void qdictionnaire::SauverFichier()
{
         QSettings settings;
         QString DernierRep=settings.value("DernierRep").toString();
         if ( DernierRep == "" ) DernierRep=".";
         if (   fichiertexte.isEmpty() )
         {
         QFileDialog FileDial;
         fichiertexte = FileDial.getSaveFileName(this, tr("sauvegarde du fichier"),
                                                        DernierRep,
                                                        "Fichiers textes(*)");
         QFileInfo fichier(fichiertexte);
         DernierRep= fichier.path();
         settings.setValue("DernierRep", DernierRep);
         AjouteListeFichier(fichiertexte);
         }
         QFile file(fichiertexte);
         if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
                 return;
         QTextStream out(&file);
         out << ui->textEditCodeEditor->toPlainText();
         EditChange=false;
}

void qdictionnaire::SauverFichierSous()
{
         QSettings settings;
         QString DernierRep=settings.value("DernierRep").toString();
         if ( DernierRep == "" ) DernierRep=".";
         QFileDialog FileDial;
         fichiertexte = FileDial.getSaveFileName(this, tr("sauvegarde du fichier"),
                                                        DernierRep,
                                                        "Fichiers textes(*)");
         QFileInfo fichier(fichiertexte);
         DernierRep= fichier.path();
         settings.setValue("DernierRep", DernierRep);
         QFile file(fichiertexte);
         if (!file.open(QIODevice::ReadWrite | QIODevice::Text))
                 return;

         QTextStream out(&file);
         out << ui->textEditCodeEditor->toPlainText();
         EditChange=false;
         AjouteListeFichier(fichiertexte);
}


void qdictionnaire::ActionNouveautes()
{
    Nouveautes *window =new Nouveautes(this);
    window->show();
}

void qdictionnaire::NouveauFichier()
{
  AskandSave();
  ui->textEditCodeEditor->Clear();
  EditChange=false;
  ui->tabWidgetMode->setCurrentIndex(Onglet::Edition);
}


void qdictionnaire ::InformationsSystemes()
{
    DialogSystemInfo dial(this);
    dial.setInfo(mPathBase);
    dial.exec();

}

void qdictionnaire::EditeurChange()
{
    EditChange=true;
}

void qdictionnaire::SauveGeometry()
{
    QSettings settings;
    if ( settings.value("WindowGeometry/SaveMainWindowGeometry").toBool())
    {
        settings.setValue("WindowGeometry/width",this->width());
        settings.setValue("WindowGeometry/height",this->height());
        settings.setValue("WindowGeometry/splitterstage",ui->splitterPrincipal->saveState());
        settings.setValue("WindowGeometry/splitterdroite",ui->splitterDroite->saveState());
        settings.setValue("WindowGeometry/splittervertical",ui->splitterVertical->saveState());
    }
}

void qdictionnaire::AuRevoir()
{
        QSettings settings;
        if ( EditChange && settings.value("SauvegarderAvantQuitter")==1)
        AskandSave();
}

 qdictionnaire::~qdictionnaire()
{}


 void qdictionnaire::EachSecondCheck()
 {
     // Teste que l'application n'est lancee qu'une seule fois
     QSettings s;
     if ( s.value("PID").toDouble() != qApp->applicationPid() )
     {
         QString PathBase=s.value("NomBase").toString();
         bool baseexist=QFileInfo(PathBase).exists();
         int basesize=QFileInfo(PathBase).size();
         bool isfile=QFileInfo(PathBase).isFile();
         if ( baseexist && basesize != 0 && isfile )
          exit(-1);
     }
 }

 void qdictionnaire::closeEvent(QCloseEvent *event)
 {
     if (mSystemTray->isVisible()) {
         hide();
         event->ignore(); // Don't let the event propagate to the base class
     }
 }

 void qdictionnaire::Activation(QSystemTrayIcon::ActivationReason raison)
 {
     if ( raison == QSystemTrayIcon::DoubleClick || raison == QSystemTrayIcon::Trigger)
         AfficheMasque();
 }



 void qdictionnaire::Licence()
 {
     Eula dial (this);
     dial.setModeShow();
     dial.exec();

 }

 void qdictionnaire::Home()
 {
    ui->tableViewListeReponse->scrollToTop();
    ui->tableViewListeReponse->selectRow(0);
    AfficheTabulation(ui->tableViewListeReponse->currentIndex());
 }

 void qdictionnaire::End()
 {
    ui->tableViewListeReponse->scrollToBottom();
    int row=ui->tableViewListeReponse->rowAt(ui->tableViewListeReponse->height());
    if (  row > 0 )
        ui->tableViewListeReponse->selectRow(row);
    else if ( row == -1 )ui->tableViewListeReponse->selectRow(m_rimeModel->m_total-1);
    AfficheTabulation(ui->tableViewListeReponse->currentIndex());
 }

 void qdictionnaire::Previous()
 {
    int row=ui->tableViewListeReponse->currentIndex().row();
    if (  row > 0 )
        ui->tableViewListeReponse->selectRow(row-1);
    AfficheTabulation(ui->tableViewListeReponse->currentIndex());
 }

 void qdictionnaire::Next()
 {
     int row=ui->tableViewListeReponse->currentIndex().row();
     int total=m_rimeModel->m_total ;
     if (  row !=total)
         ui->tableViewListeReponse->selectRow(row+1);
     AfficheTabulation(ui->tableViewListeReponse->currentIndex());


 }

 void qdictionnaire::PageNext()
 {
   int row=ui->tableViewListeReponse->currentIndex().row();
   int max=m_rimeModel->m_total;
   int nb=nbItemPerPage(ui->tableViewListeReponse);
   while (nb--  && row < max) ui->tableViewListeReponse->selectRow(++row);
   AfficheTabulation(ui->tableViewListeReponse->currentIndex());
 }

 void qdictionnaire::PagePrevious()
 {
    int row=ui->tableViewListeReponse->currentIndex().row();
    int nb=nbItemPerPage(ui->tableViewListeReponse);
    while(nb--  && row > 0) ui->tableViewListeReponse->selectRow(--row);
    AfficheTabulation(ui->tableViewListeReponse->currentIndex());
 }

 int qdictionnaire::nbItemPerPage(QTableView *ptr)
 {
    return ptr->rowAt(ptr->height())-ptr->rowAt(0);
 }


 void qdictionnaire::MasqueAfficheFleches(int , int )
 {
     bool v= ui->splitterPrincipal->sizes().at(1)<5;
     ui->toolButtonHome->setHidden(v);
     ui->toolButtonEnd->setHidden(v);
     ui->toolButtonNext->setHidden(v);
     ui->toolButtonPrevious->setHidden(v);
     ui->toolButtonPageNext->setHidden(v);
     ui->toolButtonPagePrevious->setHidden(v);
     ui->actionListe_des_rimes->disconnect();
     ui->actionType_de_rimes->disconnect();
     ui->actionListe_des_rimes->setChecked(ui->splitterPrincipal->sizes().at(1)>5);
     ui->actionType_de_rimes->setChecked(ui->splitterPrincipal->sizes().at(0)>5);
     connect (ui->actionType_de_rimes,SIGNAL(toggled(bool)),this,SLOT(SetSplitterTypeRime(bool)));
     connect (ui->actionListe_des_rimes,SIGNAL(toggled(bool)),this,SLOT(SetSplitterListeRimes(bool)));




 }

 void qdictionnaire::SetSplitterTypeRime(bool value)
 {
  QList<int> list;

  if (value)
    list<<this->width()/4<<ui->splitterPrincipal->sizes().at(1)<<width()-width()/4-ui->splitterPrincipal->sizes().at(1);
  else
    list<<0<<ui->splitterPrincipal->sizes().at(1)<<width()-ui->splitterPrincipal->sizes().at(1);
  ui->splitterPrincipal->setSizes(list);
  MasqueAfficheFleches(0,0);
 }



 void qdictionnaire::SetSplitterListeRimes(bool value)
 {
     QList<int> list;
     if (value)
     list<<ui->splitterPrincipal->sizes().at(0)<<width()/4<<width()-width()/4-ui->splitterPrincipal->sizes().at(0);
        else
     list<<ui->splitterPrincipal->sizes().at(0)<<0<<width()-ui->splitterPrincipal->sizes().at(0);
      ui->splitterPrincipal->setSizes(list);
      MasqueAfficheFleches(0,0);
 }

/*$SPECIALIZATION$*/
