/***************************************************************************
 *   Copyright (C) 2007-2008-2009 by Gilles Maire   *
 *   Gilles@lesjetesdelencre.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef DICORIME_H
#define DICORIME_H

#include <QMainWindow>
#include <QApplication>
#include <QHash>
#include <QAction>
#include <QCommandLinkButton>
#include <QSystemTrayIcon>
#include <QPrinter>
#include <QQueue>
#include <QCompleter>
#include <QStringListModel>
#include <QSortFilterProxyModel>


#include "ui_qdictionnaire.h"
#include "ui_configuration.h"
#include "ui_apropos.h"
#include "ui_aide.h"
#include "ui_Statistiques.h"
#include "ui_PrintDialog.h"
#include "ui_citation.h"
#include "configuration.h"
#include "conjugaison.h"
#include "formatage.h"
#include "correctionorthographique.h"
#include "customsqlmodel.h"

#include <QtSql>


namespace Ui {
class qdictionnaire;
}

class qdictionnaire : public QMainWindow
{
      Q_OBJECT
   public:
           enum Onglet{Definition,Citation,InfoRime,Edition,Conjugaisons};
           explicit  qdictionnaire ( QWidget* parent = 0 );
           ~qdictionnaire();
           void EffacerChampsSaufRecherche();
           QString PartieRimeChaine (QString,QString);
           QString InverseMot(QString);
           QString AppPath;
           QString WhereCondition(QString mot ,QString TypeRime, int NbSyl);
           bool OngletVerbe;
           bool debug;
           int m_LigneRimeCourante;
           QModelIndex m_ModelRimeCourante;
           QString PhonemeCherchee;
           QString NombrePhonemeCherchee;
           QString TypeRimeCherchee;
           QString m_motcherche;
           QApplication *m_application;
           void setTextLineEditCherche(QString texte);
           void setPathBase( QString path){ mPathBase=path;}
           void setTranslator( QTranslator *apptranslator,QTranslator *systemtranslator);
   private:
            Ui::Dicorime *ui;

            configuration *m_configuration;
            QDialog *m_apropos;
            QDialog *m_statistiques;
            void resizeEvent(QResizeEvent *e);

            void equalizeReponsesColumns();
            void closeEvent(QCloseEvent *event);
            QString FormateGenre(QString);
            QString FormateSyllabes(QString);
            QString FormateDefinition(QString);
            QString WhereAnagramme(QString);
            bool Approchant ( QString, QString);
            bool ExtraCaractere( QString , QString);
            int NbRimeCommun(QString ,QString );
            int NbCitations;
            int NbRimes;
            int NbLignesEditeur;
            QStringList m_ListeDerniersMots;
            Ui::PrintDialog PrintUI;
            QDialog *windowPrint;
            QDialog *winCitation;
            QString Stylise ( QString, QString );
            void AfficheRime();
            void MiseAJourHaut();
            void AfficheDefinition();
            void AfficheCitation();
            void AfficheConjugaison();
            void AfficheEditeur();
            void AjouteListeFichier(QString fichier);
            void AfficheMenuListeFichier();
            QCompleter *m_CompleterLigneCherche;
            QStringListModel *m_ModelCompleter;
            Ui::citationdialog CitationUi;
            QModelIndex ListeIndexCitation;
            QString FormateCitation ( QString phrase, QString auteur,QString rubrique);
            conjugaison conj;
            void ConstruitListeHomophones ( QString text, QString type );
            void ConstruitListeHomographes(QString text, QString type);
            void ConstruitListeAnagrammes ( QString text);
            void ConstruitListeSynonymes ( QString ortho, QString cgram, QString racine,QString genre,QString nombre);
            bool ConnecteursCitation;
            QString DefinitionVerbeConjugue ( QString codecomplet, QString verbe);
            void Debug(QString);
            QStringList HomographesList;
            QStringList HomophonesList;
            QStringList AnagrammesList;
            QString IndexDefinition;
            QString VerbeAConjuguer;
            QString fichiertexte;
            bool EditChange;
            void AddMenuEdit(QString mot);
            CorrectionOrthographique *mCorrecteur;
            QSpinBox *NbPhonemes;
            QLabel *LabelTypeRime;
            QComboBox *TypeRime;
            QComboBox *HomographesCB;
            QComboBox *HomophonesCB;
            QCheckBox *checkBoxAfficherLieux;
            QCheckBox *checkBoxAfficherPrenoms;
            QCheckBox *checkBoxAfficherAdjectifs;
            QCheckBox *checkBoxAfficherVerbes;
            QCheckBox *checkBoxAfficherSigles;
            QCheckBox *checkBoxAfficherNoms;
            QToolButton *EffacerBouton;
            QLabel *LabelMotTrouve;
            QLabel *LabelRimeaveclemot;
            QLabel *LabelNombreTrouve;
            QLabel *NombreTrouve;
            QLabel *PhonemisationLabel;
            QSystemTrayIcon *mSystemTray;
            QMenu *mMenu;
            QString mPathBase;
            CustomSqlModel *m_rimeModel;
            QSortFilterProxyModel *m_rimeProxyModel;         
            int nbItemPerPage(QTableView *ptr);
            QTranslator *m_systemtranslator;
            QTranslator *m_appltranslator;
            void OuvrirDernierFichier();
            void AskandSave();
public slots:
            void ChercherDefinition(QString index);

   public slots:
      void AfficheTabulation();

   private slots:
      void closeCitation();
       void equalizeCitationsColumns();
      void ChercherRimes(QString,int);
      void ChercherCitations(QString);
      void AskCitations(QString);
      void AskRime(QString);
      void AskConjugaison(QString);
      void AskDefinition(QString);
      void RelancerRechercheHomographes();
      void RelancerRechercheHomophones();
      void RelancerRechercheAnagramme();
      void impression();
      void EnablePrinter();
      void EffacerBouton_clicked();
      void configurationdef();
      void apropos();
      void aide();
      void DefinitionMot();
      void ChangerMode();
      void Statistiques();
      void ChercherLibreCitationAuteur();
      void TestSelection();
      void Print();
      void Preview();
      void ClosePrint();
      void AllSelectDeselect();
      void FabriquePhrase();
      void AfficherRimeParType();
      void AfficherRimeParNbPhoneme();
      void OuvrirFichier();
      void SauverFichier();
      void NouveauFichier();
      void ActionNouveautes();
      void SauverFichierSous();
      void producePrint ( QPrinter * );
      void AfficheUneCitation( QModelIndex ) ;
      void ActualiseCitation(int );
      void EditeurChange();
      void OuvrirFichierAction();
      void AffichagePhonemisation();
      void AuRevoir();
      void ReloadConfig();
      void MontreSourceWiktionary();
      void SauveGeometry();
      ///
      /// \brief EachSecondCheck : teste toutes les secondes si une seule occurence de l'application
      /// est lancée
      ///
      void EachSecondCheck();
      ///
      /// \brief AfficheMasque : cette fonction affiche ou masque la fenêtre principale
      /// car l'application est en mode systray
      ///
      void AfficheMasque();
      void Activation(QSystemTrayIcon::ActivationReason);
      void Correction();
      void InformationsSystemes();
      void Licence();
      void Home();
      void End();
      void Previous();
      void Next();
      void PageNext();
      void PagePrevious();

      void AfficheTabulation(QModelIndex index);
      ///
      /// \brief AfficheInfoRime : affiche l'onglet avec les différentes informations issues de la base lexique
      ///
      void AfficheInfoRime();
      void MasqueAfficheFleches(int i, int j);
      void SetSplitterTypeRime(bool value);
      void SetSplitterListeRimes(bool value);
      void AfficheTabulation(QModelIndex selected, QModelIndex);
      void ChangeLangs(QTranslator *systr, QTranslator *apptr);
      void AfficheTabulation(int index);
};

#endif

